using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Text;

namespace GlobalMapperDLLWrapper
{
    // Define types
    using GM_LayerHandle_t32 = IntPtr;
    using GM_FeatureClass_t16 = UInt16;
    
    // This describes a single point location
    public struct GM_Point_t
    {
        public double   mX;     // X (or longitude) coordinate
        public double   mY;     // Y (or latitude) coordinate
    };

    // This type describes a bounding rectangle in world coordinates
	public struct GM_Rectangle_t
	{
		public double  mMinX;  // Minimum x/easting/longitude coordinate
		public double  mMinY;  // Minimum y/northing/latitude coordinate
		public double  mMaxX;  // Maximum x/easting/longitude coordinate
		public double  mMaxY;  // Maximum y/northing/latitude coordinate
	};

	// Enumeration of the projection types. This enumeration is shared by the plugins
	// so it is important that the order not change and that new projections be added
	// such that the enumerated value of existing projections is not disturbed.
	public enum PROJSYS : short
	{ 
		GM_PRJ_GEO = 0,             // Geographic (lat/lon)
		GM_PRJ_UTM,                 // UTM
		GM_PRJ_SPCS,                // US State Plane
		GM_PRJ_ALBERS,              // Albers Conic
		GM_PRJ_LAMCC,               // Lambert Conformal Conic
		GM_PRJ_MERCAT,              // Mercator
		GM_PRJ_PS,                  // Polar Stereographic
		GM_PRJ_POLYC,               // Polyconic
		GM_PRJ_EQUIDC_A,            // Equidistant Conic A
		GM_PRJ_TM,                  // Transverse Mercator
		GM_PRJ_STEREO,              // Stereographic
		GM_PRJ_LAMAZ,               // Lambert Azimuthal
		GM_PRJ_AZMEQD,              // Azimuthal Equidistant
		GM_PRJ_GNOMON,              // Gnomonic
		GM_PRJ_ORTHO,               // Orthographic
		GM_PRJ_GVNSP,               // General Vertical Near Perpendicular
		GM_PRJ_SNSOID,              // Sinusoidal
		GM_PRJ_EQRECT,              // Equirectangular
		GM_PRJ_MILLER,              // Miller Cylindrical
		GM_PRJ_VGRINT,              // Van der Grinten 
		GM_PRJ_HOM_A,               // Hotine Oblique Mercator A
		GM_PRJ_ROBIN,               // Robinson
		GM_PRJ_SOM_A,               // Space Oblique Mercator A
		GM_PRJ_ALASKA,              // Alaska Conformal
		GM_PRJ_GOOD,                // Interrupted Goode
		GM_PRJ_MOLL,                // Molleweide
		GM_PRJ_IMOLL,               // Interrupted Molleweide
		GM_PRJ_HAMMER,              // Hammer-Aitoff
		GM_PRJ_WAGIV,               // Wagner IV
		GM_PRJ_WAGVII,              // Wagner VII
		GM_PRJ_OBLEQA,              // Oblated Equal Area
		GM_PRJ_EQUIDC_B,            // Equidistant Conic B
		GM_PRJ_HOM_B,               // Hotine Oblique Mercator B
		GM_PRJ_SOM_B,               // Space Oblique Mercator B
		GM_PRJ_NZMG,                // New Zealand Map Grid
		GM_PRJ_OB_STEREO,           // Oblique Stereographic
		GM_PRJ_GALL,                // Gall Stereographic
		GM_PRJ_ECKERT_IV,           // Eckert IV
		GM_PRJ_AITOFF_WAGNER,       // Aitoff-Wagner
		GM_PRJ_BONNE,               // Bonne
		GM_PRJ_CASSINI_SOLDNER,     // Cassini-Soldner
		GM_PRJ_MILOS,               // Miller Oblated Stereographic
		GM_PRJ_TIMES,               // Times
		GM_PRJ_BIPOLAR_CONIC_WEST,  // Bipolar Oblique Conic for Western Hemisphere
		GM_PRJ_2PT_EQUIDISTANT,     // Two Point Equidistant
		GM_PRJ_FIXED_SCREEN_POS,    // Fixed Location on the Screen (not a true map projection)
		GM_PRJ_LAMBERT_CYL_EQ_AREA, // Lambert Cylindrical Equal Area
		GM_PRJ_KROVAK,              // Krovak
		GM_PRJ_LAMBERT_MARTA,       // Lambert Marta (WSI Lambert)
		GM_PRJ_LAMBERT_NOAA,        // Lambert NOAA Port (Marta)
		GM_PRJ_MARTA_SATELLITE,     // Marta Satellite projection
		GM_PRJ_WINKEL_I,            // Winkel I
		GM_PRJ_POLYCONIC_INT_MAP,   // International Map of the World Polyconic
		GM_PRJ_WINKEL_TRIPEL,       // Winkel/Tripel
        GM_PRJ_LABORDE,             // Laborde (Madagascar)
        GM_PRJ_TM_SOUTH_ORIENTATED, // Transverse Mercator South Orientated
        GM_PRJ_RSO,                 // RSO (Rectified Skew Orthomorphic)
        GM_PRJ_DOUBLE_STEREOGRAPHIC,// Double Stereographic
        GM_PRJ_INT_SINUSOIDAL,      // Integerized Sinusoidal (MODIS)

		GM_PRJ_NUM_PROJ_SYS,        // Number of full projection systems

		// Projection aliases
		GM_PRJ_FIRST_ALIAS_PRJ = 0x1000,
		GM_PRJ_WTM83_91 = GM_PRJ_FIRST_ALIAS_PRJ,
		GM_PRJ_MICHIGAN_GEOREF,     	// Michigan Georef
		GM_PRJ_SWEDISH_GRID,        	// Swedish Grid
		GM_PRJ_BRITISH_GRID,        	// British Grid
		GM_PRJ_NZTM_2000,           	// New Zealand Transverse Mercator 2000
		GM_PRJ_LAMBERT_I,           	// Lambert I - North of France
		GM_PRJ_LAMBERT_II,          	// Lambert II - Central France
		GM_PRJ_LAMBERT_III,         	// Lambert III - Southern France
		GM_PRJ_LAMBERT_IV,          	// Lambert IV - Corsica
		GM_PRJ_LAMBERT_II_WIDE,     	// Lambert II - Wide
		GM_PRJ_LAMBERT_93,          	// Lambert 93
		GM_PRJ_DUTCH_GRID,          	// RijksDriehoeks (RD) - Dutch Grid
		GM_PRJ_STEREO70,                // Stereo70 (Romania)
		GM_PRJ_PLATE_CARREE,            // Plate Carree (Geographic)
		GM_PRJ_GAUSS_KRUEGER,           // Gauss Krueger (6 degree zones)
		GM_PRJ_GAUSS_KRUEGER_GERMANY,   // Gauss Krueger (3 degree zones - Germany)
		GM_PRJ_GAUSS_KRUEGER_SLOVENIA,  // Gauss Krueger (Slovenia)
		GM_PRJ_AUSTRIAN_GRID,           // Austrian Grid
		GM_PRJ_GAUSS_BOAGA,             // Gauss Boaga (Italy)
		GM_PRJ_GREEK_GRID,              // Greek Grid
		GM_PRJ_POLAND_1992_19,          // Polish Coordinate System (1992/19)
		GM_PRJ_GAUSS_KRUEGER_ARGENTINA, // Gauss Krueger (Argentina)
		GM_PRJ_SWISS_GRID,              // Swiss Grid
		GM_PRJ_IRISH_GRID,              // Irish Grid
		GM_PRJ_QUEBEC_MTM,              // Quebec MTM
		GM_PRJ_ISRAEL_GRID_OLD,         // Old Israel Grid
		GM_PRJ_ISRAEL_GRID_NEW,         // New Israel Grid
		GM_PRJ_LAMBERT_I_CARTO,         // Lambert I - North of France
		GM_PRJ_LAMBERT_III_CARTO,       // Lambert III - Southern France
		GM_PRJ_LAMBERT_IV_CARTO,        // Lambert IV - Corsica
		GM_PRJ_HUNGARIAN_EOV,           // Hungarian National Grid (EOV)
		GM_PRJ_RSO_EAST_MALAYSIA,       // East Malaysia BRSO
		GM_PRJ_RSO_WEST_MALAYSIA,       // West Malaysia RSO
		GM_PRJ_CASSINI_MALAYSIA,        // Cassini Malaysia (zoned projection)
		GM_PRJ_KOREAN_GRID,             // Korean Grid
		GM_PRJ_INDIA_GRID,              // India Grid
        GM_PRJ_BELGE_1972,              // Belge 1972 (Belgian Lambert 1972)
        GM_PRJ_MGA,                     // MGA (Map Grid Australia)
        GM_PRJ_NEW_BRUNSWICK_STEREO,    // New Brunswick Stereographic Double
        GM_PRJ_MSTM,                    // MSTM( Mississippi Transverse Mercator)
        GM_PRJ_SYSTEM_1965,             // System 1965 (Uklad 1965) (Poland)
        GM_PRJ_WISCONSIN_COUNTY,        // Wisconsin Coordinate Reference Systems (WISCRS)
        GM_PRJ_PHILIPPINES_GRID,        // Philippines Grid
        GM_PRJ_JAPANESE_GRID,           // Japanese Grid

		GM_PRJ_UNKNOWN = -1 
	};

	// Enumeration of the unit types. This enumerator is shared by plugins and
	// the Global Mapper DLL interface, so it is important that the order does
	// not change and that new units are added to the end.
	public enum UNIT : short
	{ 
		GM_PRJ_UNIT_RADIANS = 0, 
		GM_PRJ_UNIT_US_FEET, 
		GM_PRJ_UNIT_METERS, 
		GM_PRJ_UNIT_ARC_SECONDS, 
		GM_PRJ_UNIT_ARC_DEGREES, 
		GM_PRJ_UNIT_INTERNATIONAL_FEET,
		GM_PRJ_UNIT_NAUTICAL_MILES,
		GM_PRJ_UNIT_YARDS,
		GM_PRJ_UNIT_GRADES,
		GM_PRJ_UNIT_INDIAN_YARD,
        GM_PRJ_UNIT_CLARKE_FEET,
        GM_PRJ_UNIT_INCHES,

		GM_PRJ_UNIT_UNKNOWN = -1 
	};

	// Enumeration of the datum types. This enumeration is shared by the plugins
	// so it is important that the order not change and that new datums be added
	// to the end (before the interplanetary datums is ok)
	public enum DATUM : short
	{ 
		GM_DATUM_ADINDAN = 0,          
		GM_DATUM_ARC1950, 
		GM_DATUM_ARC1960, 
		GM_DATUM_AUSTRALIAN_GEODETIC_1966,
		GM_DATUM_AUSTRALIAN_GEODETIC_1984, 
		GM_DATUM_CAMP_AREA_ASTRO, 
		GM_DATUM_CAPE,
		GM_DATUM_EUROPEAN_DATUM_1950, 
		GM_DATUM_EUROPEAN_DATUM_1979, 
		GM_DATUM_GEODETIC_DATUM_1949,
		GM_DATUM_HONG_KONG_1963, 
		GM_DATUM_HU_TZU_SHAN, 
		GM_DATUM_INDIAN_1954, 
		GM_DATUM_NAD27_CONUS, 
		GM_DATUM_NAD83, 
		GM_DATUM_OLD_HAWAIIAN_MEAN, 
		GM_DATUM_OMAN, 
		GM_DATUM_ORDNANCE_SURVEY_1936, 
		GM_DATUM_PUERTO_RICO, 
		GM_DATUM_PULKOVO_1942, 
		GM_DATUM_PROVISIONAL_S_AMERICAN_1956, 
		GM_DATUM_TOKYO, 
		GM_DATUM_WGS_72, 
		GM_DATUM_WGS_84, 
		GM_DATUM_NAD27_ALASKA,
		GM_DATUM_NAD27_ALEUTIANS_EAST,
		GM_DATUM_NAD27_ALEUTIANS_WEST,
		GM_DATUM_NAD27_YPG,             // Yuma Proving Ground custom NAD27
		GM_DATUM_SWISS_GRID,            // Swiss Grid CH1903 Datum
		GM_DATUM_EGYPT_1907,            // Old Egyptian 1907
		GM_DATUM_SOUTH_AMERICA_1969,    // South America 1969
		GM_DATUM_RT90,                  // RT90 (Swedish Grid)
		GM_DATUM_GUAM_1963,             // Guam 1963
		GM_DATUM_POTSDAM,               // Potsdam (Germany) datum
		GM_DATUM_MGI_AUSTRIA,           // MGI Austria datum
		GM_DATUM_GDA94,                 // Geocentric Datum of Australia 1994
		GM_DATUM_INDIAN_1975,           // Indian 1975
		GM_DATUM_NZGD_2000,             // New Zealand Geodetic Datum 2000
		GM_DATUM_NAHRWAN,               // Nahrwan 1929 (United Arab Emirates)
		GM_DATUM_S_42_HUNGARY,          // S-42 (Pulkovo 1942) - Hungary
		GM_DATUM_S_42_POLAND,           // S-42 (Pulkovo 1942) - Poland
		GM_DATUM_S_42_CZECH,            // S-42 (Pulkovo 1942) - Czechoslavakia
		GM_DATUM_S_42_LATVIA,           // S-42 (Pulkovo 1942) - Latvia
		GM_DATUM_S_42_KAZAKHSTAN,       // S-42 (Pulkovo 1942) - Kazakhstan
		GM_DATUM_S_42_ALBANIA,          // S-42 (Pulkovo 1942) - Albania
		GM_DATUM_S_42_ROMANIA,          // S-42 (Pulkovo 1942) - Romania
		GM_DATUM_D_48_SLOVENIA,         // D-48 - Slovenia
		GM_DATUM_NAD27_CANADA,          // NAD27 (mean over Canada)
		GM_DATUM_MINNA_CAMEROON,        // Minna - Cameroon
		GM_DATUM_MINNA_NIGERIA,         // Minna - Nigeria
		GM_DATUM_DUTCH,                 // Dutch RD (Netherlands)
		GM_DATUM_ZANDERIJ,              // Zanderij - Suriname
		GM_DATUM_FINLAND_HAYFORD,       // Finland/Hayford
		GM_DATUM_BEIJING_1954,          // Beijing 1954
		GM_DATUM_NGO_1948,              // NGO 1948 (Oslo)
		GM_DATUM_NTF_FRANCE_GREENWICH,  // NTF France (Greenwich Meridian)
		GM_DATUM_CHAD_WG1998,           // Chad WG1998
		GM_DATUM_HJORSEY_1955,          // Hjorsey 1955
		GM_DATUM_BOGOTA_OBSERVATORY,    // Bogota Observatory
		GM_DATUM_SCHWARZECK,            // Schwarzeck - Namibia
		GM_DATUM_NORTH_SAHARA_1959,     // North Sahara 1959
		GM_DATUM_REUNION_ISLAND,        // Reunion Island
		GM_DATUM_IRELAND_1965,          // Ireland 1965
		GM_DATUM_AIN_EL_ABD_BAHRAIN,    // Ain El Abd (Bahrain)
		GM_DATUM_AIN_EL_ABD_SAUDI_ARABIA,// Ain El Abd (Saudi Arabia)
		GM_DATUM_LUZON,                 // Luzon (Phillipines)
		GM_DATUM_CARTHAGE,              // Carthage (Tunisia)
		GM_DATUM_MERCHICH,              // Merchich (Morocco)
		GM_DATUM_NAD27_MEXICO,          // NAD27 (mean over Mexico)
		GM_DATUM_ED50_ENGLAND,          // ED50 - England (EUR_K)
		GM_DATUM_ED50_FINLAND_NORWAY,   // ED50 - Finland and Norway (EUR_C)
		GM_DATUM_ED50_GREECE,           // ED50 - Greece (EUR_B)
		GM_DATUM_ED50_SICILY,           // ED50 - Italy (Sicily) (EUR_J)
		GM_DATUM_ED50_SPAIN_PORTUGAL,   // ED50 - Spain and Portugal (EUR_D)
		GM_DATUM_ROME_1940,             // Rome 1940
		GM_DATUM_KERTAU_1948,           // Kertau 1948
		GM_DATUM_LEIGON,                // Leigon (Ghana)
		GM_DATUM_TIMBALAI_1948,         // Timbalai 1948
		GM_DATUM_ASTRO_B4_SOROL_ATOLL,  // Astro B4 (Sorol Atoll)
		GM_DATUM_ASTRO_BEACON,          // Astro Beacon
		GM_DATUM_ASTRO_DOS_71_4,        // Astro DOS 71/4
		GM_DATUM_ASTRONOMIC_STN_1952,   // Astronomic Station 1952
		GM_DATUM_EASTER_ISLAND_1967,    // Easter Island 1967
		GM_DATUM_HERMANNSKOGEL,         // Hermannskogel
		GM_DATUM_INDIAN_BANGLADESH,     // Indian (Bangladesh)
		GM_DATUM_INDIAN_MEAN_VALUE,     // Indian (Mean)
		GM_DATUM_INDIAN_THAILAND,       // Indian (Thailand)
		GM_DATUM_ISTS_073_ASTRO,        // ISTS 073 Astro
		GM_DATUM_NAD27_BAHAMAS,         // NAD27 (Bahamas)
		GM_DATUM_NAD27_CANAL_ZONE,      // NAD27 (Canal Zone)
		GM_DATUM_NAD27_CARIBBEAN,       // NAD27 (Caribbean)
		GM_DATUM_NAD27_CENTRAL,         // NAD27 (Central America)
		GM_DATUM_NAD27_CUBA,            // NAD27 (Cuba)
		GM_DATUM_NAD27_GREENLAND,       // NAD27 (Greenland)
		GM_DATUM_NAD27_SAN_SALVADOR,    // NAD27 (San Salvador)
		GM_DATUM_OBSERVATORIO_1966,     // Observatorio_1966
		GM_DATUM_PICO_DE_LAS_NIEVES,    // Pico de las Nieves
		GM_DATUM_PORTUGESE_1973,        // Portugese 1973
		GM_DATUM_PROV_S_CHILEAN,        // Provisional South Chilean (1963)
		GM_DATUM_QORNOQ,                // Qornoq
		GM_DATUM_SAO_BRAZ,              // Sao Braz
		GM_DATUM_SOUTHEAST_BASE,        // Southeast Base
		GM_DATUM_SOUTHWEST_BASE,        // Southwest Base
		GM_DATUM_KKJ,                   // KKJ (Finnish Grid)
		GM_DATUM_GGRS87,                // GGRS 1987 (Greek Grid)
		GM_DATUM_POINTE_NOIRE,          // Pointe Noire 1948 (Congo)
		GM_DATUM_RGF_93,                // RGF-93 (France)
		GM_DATUM_CAMPO_INCHAUSPE,       // Campo Inchauspe (Argentina)
		GM_DATUM_ETRS89,                // ETRS89 (coincident with WGS-84 within a meter)
		GM_DATUM_NTF_FRANCE_PARIS,      // NTF France (Pantheon in Paris Meridian)
		GM_DATUM_HARTEBEESTHOEK94,      // Hartebeesthoek94 (South Africa)
		GM_DATUM_INDONESIA_1974,        // Indonesia 1974
		GM_DATUM_BUKIT_RIMPAH,          // Bukit Rimpah
		GM_DATUM_NAD27,                 // NAD27 - auto-selects best NAD27 specialization
		GM_DATUM_LUREF,                 // Luxembourg Reference Datum
		GM_DATUM_QND95,                 // Qatar National Datum 1995
		GM_DATUM_OLD_ISRAELI,           // Old Israeli Datum
		GM_DATUM_NEW_ISRAELI,           // New Israeli Datum
		GM_DATUM_CORREGO_ALEGRE,        // Corrego Alegre (Brazil)
		GM_DATUM_POSGAR94,              // Posgar 94  datum (Argentina)  
		GM_DATUM_HD72,                  // Hungarian datum 1972
		GM_DATUM_ATS77_NOVA_SCOTIA,     // ATS77 (Average Terrestrial System 1977) - Nova Scotia
		GM_DATUM_INDIAN_NEPAL,          // India (Nepal) - Everest 1956 ellipsoid
		GM_DATUM_YACARE,                // Yacare (Uruguay)
		GM_DATUM_TANANARIVE_OBS_1925,   // Tananarive Observatory 1925 (Madagascar)
		GM_DATUM_S_JTSK,                // S-JTSK (Czech and Slovak Republics)
		GM_DATUM_GRS80_ELLIPSOID,       // GRS80 Ellipsoid (equivalent to WGS84, included for confused users)
		GM_DATUM_KGD,                   // Korean Geodetic Datum (KGD)
		GM_DATUM_IRELAND_1975,          // Ireland 1965 with 1975 Adjustment
		GM_DATUM_DP1970,                // Dealul Piscului 1970
		GM_DATUM_LISBON_DLX,            // Lisbon (DLx)
		GM_DATUM_SOLOMON_1968,          // Solomon 1968 (GUX 1 Astro)
		GM_DATUM_M_PORALOKO,            // M'Poraloko (Gabon)
		GM_DATUM_GOOGLE_MAPS,           // Google Maps datum (Sphere Radius 6378137 - has special relation to WGS84)
		GM_DATUM_BD72,                  // BD72 (Belgium 1972)
        GM_DATUM_NGO_1948_OSLO,         // NGO 1948 (Oslo Meridian)
        GM_DATUM_TANANARIVE_1925_PARIS, // Tananarive Observatory 1925 (Madagascar) (Paris PM)
        GM_DATUM_ARATU,                 // Aratu (Brazil Oil Exploration)
        GM_DATUM_CAMACUPA,              // Camacupa
        GM_DATUM_JAD69,                 // JAD69 (Jamaica 1969)
        GM_DATUM_JAD2001,               // JAD2001 (Jamaica 2001) [Equivalent to WGS84]
        GM_DATUM_EUROPEAN_DATUM_1987,   // ED87 (European Datum 1987)
        GM_DATUM_S_42_AZ_GEORGIA,       // S-42 (Pulkovo 1942) (Azerbaijan and Georgia)
        GM_DATUM_S_42_ESTONIA,          // S-42 (Pulkovo 1942) (Estonia)
        GM_DATUM_S_42_GERMANY,          // S-42 (Pulkovo 1942) (Germany - Mecklenburg-Vorpommern and Sachsen-Anhalt)
        GM_DATUM_S_42_LITHUANIA,        // S-42 (Pulkovo 1942) (Lithuania)
        GM_DATUM_S_42_RUSSIA,           // S-42 (Pulkovo 1942) (Russia)
        GM_DATUM_IRENET95,              // Ireland IRENET95 datum
        GM_DATUM_PRS92,                 // Philippine Reference System 1992
        GM_DATUM_MALONGO_1987,          // Malongo 1987
        GM_DATUM_SWEREF99,              // SWEREF99
        GM_DATUM_KOREAN_1985,           // Korean 1985
        GM_DATUM_KOREAN_1995,           // Korean 1995
        GM_DATUM_HERAT_NORTH,           // Herat North
        GM_DATUM_DEIR_EZ_ZOR,           // Deir ez Zor (Syria)
        GM_DATUM_EGYPT_GULF_OF_SUEZ,    // Egypt Gulf of Suez S-650 TL
        GM_DATUM_JORDAN,                // Jordan
        GM_DATUM_FD_1958,               // FD 1958 (Iraq)
        GM_DATUM_OCOTEPEQUE,            // Ocotepeque (Costa Rica)
        GM_DATUM_ISN93,                 // ISN93 (Island Network 1993) - Iceland
        GM_DATUM_REYKJAVIK_1900,        // Reykjavik 1900
        GM_DATUM_ESTONIA_1997,          // Estonia 1997
        GM_DATUM_TRINIDAD_1903,         // Trinidad 1903
        GM_DATUM_LKS92,                 // LKS92 (Latvia)
        GM_DATUM_MODIS_SPHEROID,        // Spheroid Used by MODIS Imagery
        GM_DATUM_ETRF_2000,             // ETRF2000 - RDN
        GM_DATUM_ROME_1940_ROME,        // Rome 1940 (Monte Mario) - Rome Meridian
        GM_DATUM_NAPARIMA_1972,         // Naparima 1972 (Trinidad & Tobago)

		// Child datums
		GM_DATUM_CHILD_FIRST = 0x1000,
		GM_DATUM_NAD83_ALABAMA,         // Alabama HPGN
		GM_DATUM_NAD83_ARIZONA,         // Arizona HPGN
		GM_DATUM_NAD83_ARKANSAS,        // Arkansas HPGN
		GM_DATUM_NAD83_CALIFORNIA_NORTH,// California - North HPGN
		GM_DATUM_NAD83_CALIFORNIA_SOUTH,// California - South HPGN
		GM_DATUM_NAD83_COLORADO,        // Colorado HPGN
		GM_DATUM_NAD83_FLORIDA,         // Florida HPGN
		GM_DATUM_NAD83_GEORGIA,         // Georgia HPGN
		GM_DATUM_NAD83_HAWAII,          // Hawaii HPGN
		GM_DATUM_NAD83_ILLINOIS,        // Illinois HPGN
		GM_DATUM_NAD83_INDIANA,         // Indiana HPGN
		GM_DATUM_NAD83_IOWA,            // Iowa HPGN
		GM_DATUM_NAD83_KANSAS,          // Kansas HPGN
		GM_DATUM_NAD83_KENTUCKY,        // Kentucky HPGN
		GM_DATUM_NAD83_LOUISIANA,       // Louisiana HPGN
		GM_DATUM_NAD83_MAINE,           // Maine HPGN
		GM_DATUM_NAD83_MARYLAND,        // Maryland-Delaware HPGN
		GM_DATUM_NAD83_MICHIGAN,        // Michigan HPGN
		GM_DATUM_NAD83_MINNESOTA,       // Minnesota HPGN
		GM_DATUM_NAD83_MISSISSIPPI,     // Mississippi HPGN
		GM_DATUM_NAD83_MISSOURI,        // Missouri HPGN
		GM_DATUM_NAD83_MONTANA_EAST,    // Montana East HPGN
		GM_DATUM_NAD83_MONTANA_WEST,    // Montana West/Idaho HPGN
		GM_DATUM_NAD83_NEBRASKA,        // Nebraska HPGN
		GM_DATUM_NAD83_NEVADA,          // Nevada HPGN
		GM_DATUM_NAD83_NEW_ENGLAND,     // New England HPGN
		GM_DATUM_NAD83_NEW_JERSEY,      // New Jersey HPGN
		GM_DATUM_NAD83_NEW_MEXICO,      // New Mexico HPGN
		GM_DATUM_NAD83_NEW_YORK,        // New York HPGN
		GM_DATUM_NAD83_NORTH_DAKOTA,    // North Dakota HPGN
		GM_DATUM_NAD83_OHIO,            // Ohio HPGN
		GM_DATUM_NAD83_OKLAHOMA,        // Oklahoma HPGN
		GM_DATUM_NAD83_PENNSYLVANIA,    // Pennsylvania HPGN
		GM_DATUM_NAD83_PUERTO_RICO,     // Puerto Rico/Virgin Islands HPGN
		GM_DATUM_NAD83_SOUTH_DAKOTA,    // South Dakota HPGN
		GM_DATUM_NAD83_TENNESSEE,       // Tennessee HPGN
		GM_DATUM_NAD83_TEXAS_EAST,      // Texas East HPGN
		GM_DATUM_NAD83_TEXAS_WEST,      // Texas West HPGN
		GM_DATUM_NAD83_UTAH,            // Utah HPGN
		GM_DATUM_NAD83_VIRGINIA,        // Virginia HPGN
		GM_DATUM_NAD83_WASHINGTON,      // Washington-Oregon HPGN
		GM_DATUM_NAD83_WEST_VIRGINIA,   // West Virginia HPGN
		GM_DATUM_NAD83_WISCONSIN,       // Wisconsin HPGN
		GM_DATUM_NAD83_WYOMING,         // Wyoming HPGN
        GM_DATUM_MN_ANOKA,              // Minnesota County Datum
        GM_DATUM_MN_BECKER,             // Minnesota County Datum
        GM_DATUM_MN_BELTRAMI_NORTH,     // Minnesota County Datum
        GM_DATUM_MN_BELTRAMI_SOUTH,     // Minnesota County Datum
        GM_DATUM_MN_BENTON,             // Minnesota County Datum
        GM_DATUM_MN_BIG_STONE,          // Minnesota County Datum
        GM_DATUM_MN_BLUE_EARTH,         // Minnesota County Datum
        GM_DATUM_MN_BROWN,              // Minnesota County Datum
        GM_DATUM_MN_CARLTON,            // Minnesota County Datum
        GM_DATUM_MN_CARVER,             // Minnesota County Datum
        GM_DATUM_MN_CASS_NORTH,         // Minnesota County Datum
        GM_DATUM_MN_CASS_SOUTH,         // Minnesota County Datum
        GM_DATUM_MN_CHIPPEWA,           // Minnesota County Datum
        GM_DATUM_MN_CHISAGO,            // Minnesota County Datum
        GM_DATUM_MN_COOK_NORTH,         // Minnesota County Datum
        GM_DATUM_MN_COOK_SOUTH,         // Minnesota County Datum
        GM_DATUM_MN_COTTONWOOD,         // Minnesota County Datum
        GM_DATUM_MN_CROW_WING,          // Minnesota County Datum
        GM_DATUM_MN_DAKOTA,             // Minnesota County Datum
        GM_DATUM_MN_DODGE,              // Minnesota County Datum
        GM_DATUM_MN_DOUGLAS,            // Minnesota County Datum
        GM_DATUM_MN_FARIBAULT,          // Minnesota County Datum
        GM_DATUM_MN_FILLMORE,           // Minnesota County Datum
        GM_DATUM_MN_FREEBORN,           // Minnesota County Datum
        GM_DATUM_MN_GOODHUE,            // Minnesota County Datum
        GM_DATUM_MN_GRANT,              // Minnesota County Datum
        GM_DATUM_MN_HENNEPIN,           // Minnesota County Datum
        GM_DATUM_MN_HOUSTON,            // Minnesota County Datum
        GM_DATUM_MN_ISANTI,             // Minnesota County Datum
        GM_DATUM_MN_ITASCA_NORTH,       // Minnesota County Datum
        GM_DATUM_MN_ITASCA_SOUTH,       // Minnesota County Datum
        GM_DATUM_MN_JACKSON,            // Minnesota County Datum
        GM_DATUM_MN_KANABEC,            // Minnesota County Datum
        GM_DATUM_MN_KANDIYOHI,          // Minnesota County Datum
        GM_DATUM_MN_KITTSON,            // Minnesota County Datum
        GM_DATUM_MN_KOOCHICHING,        // Minnesota County Datum
        GM_DATUM_MN_LAC_QUI_PARLE,      // Minnesota County Datum
        GM_DATUM_MN_LAKE_OF_THE_WOODS_NORTH, // Minnesota County Datum
        GM_DATUM_MN_LAKE_OF_THE_WOODS_SOUTH, // Minnesota County Datum
        GM_DATUM_MN_LE_SUEUR,           // Minnesota County Datum
        GM_DATUM_MN_LINCOLN,            // Minnesota County Datum
        GM_DATUM_MN_LYON,               // Minnesota County Datum
        GM_DATUM_MN_MCLEOD,             // Minnesota County Datum
        GM_DATUM_MN_MAHNOMEN,           // Minnesota County Datum
        GM_DATUM_MN_MARSHALL,           // Minnesota County Datum
        GM_DATUM_MN_MARTIN,             // Minnesota County Datum
        GM_DATUM_MN_MEEKER,             // Minnesota County Datum
        GM_DATUM_MN_MORRISON,           // Minnesota County Datum
        GM_DATUM_MN_MOWER,              // Minnesota County Datum
        GM_DATUM_MN_MURRAY,             // Minnesota County Datum
        GM_DATUM_MN_NICOLLET,           // Minnesota County Datum
        GM_DATUM_MN_NOBLES,             // Minnesota County Datum
        GM_DATUM_MN_NORMAN,             // Minnesota County Datum
        GM_DATUM_MN_OLMSTED,            // Minnesota County Datum
        GM_DATUM_MN_OTTERTAIL,          // Minnesota County Datum
        GM_DATUM_MN_PENNINGTON,         // Minnesota County Datum
        GM_DATUM_MN_PINE,               // Minnesota County Datum
        GM_DATUM_MN_PIPESTONE,          // Minnesota County Datum
        GM_DATUM_MN_POLK,               // Minnesota County Datum
        GM_DATUM_MN_POPE,               // Minnesota County Datum
        GM_DATUM_MN_RAMSEY,             // Minnesota County Datum
        GM_DATUM_MN_RED_LAKE,           // Minnesota County Datum
        GM_DATUM_MN_REDWOOD,            // Minnesota County Datum
        GM_DATUM_MN_RENVILLE,           // Minnesota County Datum
        GM_DATUM_MN_RICE,               // Minnesota County Datum
        GM_DATUM_MN_ROCK,               // Minnesota County Datum
        GM_DATUM_MN_ROSEAU,             // Minnesota County Datum
        GM_DATUM_MN_ST_LOUIS_NORTH,     // Minnesota County Datum
        GM_DATUM_MN_ST_LOUIS_CENTRAL,   // Minnesota County Datum
        GM_DATUM_MN_ST_LOUIS_SOUTH,     // Minnesota County Datum
        GM_DATUM_MN_SCOTT,              // Minnesota County Datum
        GM_DATUM_MN_SHERBURNE,          // Minnesota County Datum
        GM_DATUM_MN_SIBLEY,             // Minnesota County Datum
        GM_DATUM_MN_STEARNS,            // Minnesota County Datum
        GM_DATUM_MN_STEELE,             // Minnesota County Datum
        GM_DATUM_MN_STEVENS,            // Minnesota County Datum
        GM_DATUM_MN_SWIFT,              // Minnesota County Datum
        GM_DATUM_MN_TODD,               // Minnesota County Datum
        GM_DATUM_MN_TRAVERSE,           // Minnesota County Datum
        GM_DATUM_MN_WABASHA,            // Minnesota County Datum
        GM_DATUM_MN_WADENA,             // Minnesota County Datum
        GM_DATUM_MN_WASECA,             // Minnesota County Datum
        GM_DATUM_MN_WATONWAN,           // Minnesota County Datum
        GM_DATUM_MN_WINONA,             // Minnesota County Datum
        GM_DATUM_MN_WRIGHT,             // Minnesota County Datum
        GM_DATUM_MN_YELLOW_MEDICINE,    // Minnesota County Datum
        GM_DATUM_WI_SHEBOYGAN,          // Wisconsin County Datum
        GM_DATUM_ED50_ITALY,            // ED50 - Italy
        GM_DATUM_NAD83_SOUTH_CAROLINA,  // South Carolina HPGN
        GM_DATUM_ED50_NORTH_SEA,        // ED50 - North Sea
        GM_DATUM_ED50_NORWAY_N62,       // ED50 - Norway Offshore - North of 62N
        GM_DATUM_ED50_NORWAY_S62,       // ED50 - Norway Offshore - South of 62N
        GM_DATUM_WI_BURNETT,            // Wisconsin County Datum
        GM_DATUM_WI_DOUGLAS,            // Wisconsin County Datum

		// Interplanetary datums
		GM_DATUM_IP_FIRST = 0x4000,
		GM_DATUM_IP_MERCURY = GM_DATUM_IP_FIRST,
		GM_DATUM_IP_VENUS,
		GM_DATUM_IP_VENUS_1985,
		GM_DATUM_IP_MOON,
		GM_DATUM_IP_MARS_2001,
		GM_DATUM_IP_MARS_1990,
		GM_DATUM_IP_JUPITER,
		GM_DATUM_IP_IO,
		GM_DATUM_IP_EUROPA,
		GM_DATUM_IP_GANYMEDE,
		GM_DATUM_IP_CALLISTO,
		GM_DATUM_IP_SATURN,
		GM_DATUM_IP_TITAN,
		GM_DATUM_IP_URANUS,
		GM_DATUM_IP_NEPTUNE,
		GM_DATUM_IP_PLUTO,

		// Special datum values
		GM_DATUM_UNKNOWN = -1, 
		GM_DATUM_NONE = -2, 
		GM_DATUM_DEFAULT = -3,
	};

	// Enumeration of projection attribute types. These values are shared with
	// the Global Mapper DLL so it is important that their values to not change.
	public enum PROJATTR : short
	{ 
		FIRST_STANDARD_PARALLEL = 0,
		SECOND_STANDARD_PARALLEL,
		CENTRAL_LATITUDE,
		CENTRAL_LONGITUDE,
		FALSE_EASTING,
		FALSE_NORTHING,
		SPHERE_RADIUS,
		ORIGIN_LATITUDE,
		SEMI_MAJOR_AXIS,
		SEMI_MINOR_AXIS,
		CENTRAL_MERIDIAN,
		STANDARD_PARALLEL,
		TRUE_SCALE_LATITUDE,
		AZIMUTH_ANGLE,
		AZIMUTH_POINT,
		LATITUDE1,
		LATITUDE2,
		LONGITUDE1,
		LONGITUDE2,
		PROJ_CENTER_SCALE_FACTOR,
		LONGITUDE_POLE,
		CENTRAL_MERIDIAN_SCALE_FACTOR,
		PERSPECTIVE_POINT_HEIGHT,
		INCLINATION_ANGLE,
		ASCENTION_LONGITUDE,
		SAT_REV_PERIOD,
		END_OF_PATH_FLAG,
		LANDSAT_SAT_NUM,
		OEA_SHAPE_PARM_M,
		OEA_SHAPE_PARM_N,
		OEA_OVAL_ROTATION_ANGLE,
		ZONE,
		GM_PRJ_ATTR_STATE_PLANE_SCALE_FACTOR,   // extra scale factor for State Plane projections

		// Attributes for GM_PRJ_FIXED_SCREEN_POS projection
		GM_PRJ_ATTR_HORZ_OFFSET,        // horizontal offset value in inches, percent, or itude
		GM_PRJ_ATTR_HORZ_OFFSET_TYPE,   // horizontal offset type (0 - left, 1 - right, 2 - centered)
		GM_PRJ_ATTR_HORZ_OFFSET_VTYPE,  // horizontal offset value type (0 - inches, 1 - percent, 2 - degrees itude)
		GM_PRJ_ATTR_VERT_OFFSET,        // vertical offset value in inches, percent, or latitude
		GM_PRJ_ATTR_VERT_OFFSET_TYPE,   // vertical offset type (0 - top, 1 - bottom, 2 - centered)
		GM_PRJ_ATTR_VERT_OFFSET_VTYPE,  // vertical offset value type (0 - inches, 1 - percent, 2 - degrees latitude)
		GM_PRJ_ATTR_SIZE_VALUE,         // size value (percent)
		GM_PRJ_ATTR_SIZE_VALUE_TYPE,    // size value type (0 - fixed pixel size, 1 - percent width, 2 - percent height)
        GM_PRJ_ATTR_ALWAYS_ON_TOP,      // make this fixed screen file show on top of everything else

        // Additional attributes
        GM_PRJ_ATTR_STATE_PLANE_EXTRA_FALSE_EASTING,    // extra false easting (in meters) for State Plane projections to apply AFTER the extra scale factor
        GM_PRJ_ATTR_STATE_PLANE_EXTRA_FALSE_NORTHING,   // extra false northing (in meters) for State Plane projections to apply AFTER the extra scale factor
        GM_PRJ_ATTR_STATE_PLANE_SCALE_CENTER_EASTING,   // easting of location at which to apply extra scale factor for State Plane projections
        GM_PRJ_ATTR_STATE_PLANE_SCALE_CENTER_NORTHING,  // northing of location at which to apply extra scale factor for State Plane projections

		NUM_PROJ_ATTR_TYPES
	};

	// This type is used to describe a single projection attribute value
	public struct GM_ProjAttrValue_t
	{
		public PROJATTR    mAttr;      // Attribute
		public double      mValue;     // Attribute value
	};

	// This type is used to fully describe a projection.
	public struct GM_Projection_t
	{
		public PROJSYS    		  mProjSys;		// Projection system
		public DATUM      		  mDatum;		// Horizontal datum
		public UNIT       		  mUnit;		// Ground units
		public UInt32     		  mNumAttrs;	// Number of attributes in attribute list
		public GM_ProjAttrValue_t mAttr1;		// Attribute value list items (C# doesn't support fixed length arrays)
		public GM_ProjAttrValue_t mAttr2;  
		public GM_ProjAttrValue_t mAttr3;  
		public GM_ProjAttrValue_t mAttr4;  
		public GM_ProjAttrValue_t mAttr5;  
		public GM_ProjAttrValue_t mAttr6;  
		public GM_ProjAttrValue_t mAttr7;  
		public GM_ProjAttrValue_t mAttr8;     
		public GM_ProjAttrValue_t mAttr9;  
		public GM_ProjAttrValue_t mAttr10; 
		public GM_ProjAttrValue_t mAttr11; 
		public GM_ProjAttrValue_t mAttr12; 
		public GM_ProjAttrValue_t mAttr13; 
		public GM_ProjAttrValue_t mAttr14; 
		public GM_ProjAttrValue_t mAttr15; 
		public GM_ProjAttrValue_t mAttr16; 
	};
    
    /* ----- Area feature classifications ----- */
    public enum AreaFeatureClass_t16 : ushort
    {
        AFC_MIN  = 0,
	    AFC_NONE = 0,

        AFC_ISLAND           = 1,
        AFC_RESERVATION      = 2,

        // Non-vegetative cover
        AFC_SNOW_OR_GLACIER  = 3,
        AFC_SALT             = 4,
        AFC_SAND             = 5,
        AFC_ROCK             = 6,
        AFC_REEF             = 7,

        // City metropolitan area outlines
        AFC_MIN_CITY_MA       = 10,   
        AFC_CITY_LARGE        = 10,
        AFC_CITY_SMALL        = 11,
        AFC_CITY_UNKNOWN      = 12,
        AFC_MAX_CITY_MA       = 12,

        // Coverage/Quad areas
        AFC_COVERAGE         = 15,
        AFC_MAP_CATALOG_LAYER= 16,

        // Park classifications
        AFC_MIN_PARK         = 20,
        AFC_NTNL_PARK_MJR    = 20,
        AFC_NTNL_PARK_OTHR   = 22,
        AFC_STATE_PARK_MJR   = 23,
        AFC_STATE_PARK_OTHR  = 25,
        AFC_URBAN_PARK       = 26,
        AFC_MAX_PARK         = 26,
        
        // Vegetative and non-vegetative coverage areas
        AFC_WETLAND          = 33,
        AFC_TUNDRA           = 34,
        AFC_WOODS            = 35,
        AFC_SCRUB            = 36,
        AFC_ORCHARD          = 37, // Orchard, plantation, vineyard, etc.
        AFC_CROPLAND         = 38,
        AFC_SCATTERED_TREES  = 39,

        // Water areas
        AFC_MIN_WATER           = 40,
        AFC_OCEAN               = 40,
        AFC_STILL_WATER_MJR     = 60,
        AFC_STILL_WATER_250MI   = 61,
        AFC_STILL_WATER_100MI   = 62,
        AFC_STILL_WATER_50MI    = 63,
        AFC_STILL_WATER_10MI    = 64,
        AFC_STILL_WATER_5MI     = 65,
        AFC_STILL_WATER_1MI     = 66,
        AFC_STILL_WATER_HALF    = 67,
        AFC_STILL_WATER_LT_HALF = 68,
        AFC_STILL_WATER_UNK     = 69,
        AFC_FLOW_WATER_MJR      = 70,
        AFC_FLOW_WATER_RGNL     = 71,
        AFC_FLOW_WATER_MNR      = 72,
        AFC_FLOW_WATER_INT      = 73, // Intermittent flowing water

        AFC_STILL_WATER_INT     = 75, // Intermittent lake, reservoir
        AFC_STILL_WATER_DRY     = 76, // Dry lake, reservoir
        AFC_MAX_WATER           = 76,

        // Political areas, should be drawn only in borders
        AFC_MIN_POLITICAL       = 77,
        AFC_LAND_GRANT          = 77,
        AFC_COUNTY              = 78,
        AFC_STATE               = 79,
        AFC_COUNTRY             = 80,
        AFC_SURVEY_SECTION      = 81,
        AFC_COUNTY_SUBDIVISION  = 95,
        AFC_MAX_POLITICAL       = 81,

        // Marine areas
        AFC_ISOBATH             = 85, // Isobath area
        AFC_ANCHORAGE           = 86, // Anchorage area
        AFC_LAND                = 87, // Land area
        AFC_INTERTIDAL_AREA     = 88, // Intertidal area
        AFC_MARINE_INFO_AREA    = 89, // Marine info area (i.e. danger, caution, dumping, etc.)
        AFC_MARINE_TRAFFIC_ZONE = 91, // Marine traffic zone

        // Other area types
        AFC_MISC_STRUCTURE      = 82, // Misc manmade structure
        AFC_MEASUREMENT         = 83, // Measurement
        AFC_ISOHEIGHT           = 84, // Iso-height area
        AFC_RANGE_RING          = 92, // Range ring
        AFC_VIEW_SHED_COVERAGE  = 93, // View Shed coverage area
        AFC_TIN_FACE            = 94, // Face polygon for TIN (triangulated irregular network)

        // Unknown area type
        AFC_UNKNOWN             = 90,

        AFC_MAX_BUILTIN         = 95,

        AFC_FIRST_CUSTOM        = 0x200, // First custom type number  

        AFC_INVALID             = 0xFFFF
    };

    /* ----- Line feature classifications ----- */
    public enum LineFeatureClass_t16 : ushort
    {
        LFC_MIN_LINE         = 0,
        LFC_NONE             = 0,
        
        // Road line features
        LFC_MIN_ROAD         =  1,
        LFC_INTERSTATE       =  1,
        LFC_MAJOR_HWY        =  2,
        LFC_STATE_HWY        =  3,
        LFC_ARTERIAL         =  4,
        LFC_COLLECTOR        =  5,
        LFC_RESIDENTIAL      =  6,
        LFC_ALLEY            =  7,
        LFC_RAMP             =  8,
        LFC_UNPAVED          = 10,
        LFC_COUNTY_ROUTE     = 12,
        LFC_MAX_ROAD         = 12,
        
        // Streams
        LFC_STREAM           = 15,
        LFC_STREAM_INT       = 16, /* INTERMITTENT STREAM */
        LFC_STREAM_DRY       = 17, /* DRY STREAM */

        // Railroad features
        LFC_MIN_RAILROAD     = 18,
        LFC_RAILROAD_CLASS1A = 18, /* CLASS 1A RAILROAD                        */
        LFC_RAILROAD_CLASS1B = 19, /* CLASS 1B RAILROAD                        */
        LFC_RAILROAD_OTHER   = 20, /* OTHER RAILROAD CLASSIFICATION            */
        LFC_MAX_RAILROAD     = 20,

        // Other line feature types
        LFC_SHORELINE        = 21,
        LFC_TRAIL            = 22,
        LFC_DATABASE         = 23,
        LFC_GRID_LINE        = 24,  /* EVERY GRID LINE                          */
        LFC_GRID_LINE_MINOR  = 25,  /* MINOR GRID LINES                         */
        LFC_GRID_LINE_MAJOR  = 26,  /* MAJOR GRID LINES                         */
        LFC_LOCAL_FERRY      = 27,
        LFC_MJR_PLTCL        = 28,  /* MAJOR ADMIN POLITICAL BOUNDARY (STATE)   */
        LFC_MNR_PLTCL        = 29,  /* MINOR ADMIN POLITICAL BOUNDARY (COUNTY)  */
        LFC_INTRN_PLTCL      = 30,  /* INTERNATIONAL POLITICAL BOUNDRARY        */
        LFC_CITY_BDRY        = 31,  /* CITY BOUNDARY                            */
        LFC_INVISIBLE_BDRY   = 32,  /* INVISIBLE BOUNDARY                       */
        LFC_POWERLINE        = 33,  /* POWERLINE                                */
        LFC_PARK_BOUNDARY    = 34,  /* NATIONAL PARK BOUNDARY                   */
        LFC_PIPELINE         = 35,  /* PIPELINE                                 */

        // Contour lines
        LFC_MIN_CONTOUR      = 36,
        LFC_CONTOUR          = 36,  /* EVERY CONTOUR LINES                      */
        LFC_CONTOUR_MINOR    = 37,  /* MINOR CONTOUR LINES                      */
        LFC_CONTOUR_MAJOR    = 38,  /* MAJOR CONTOUR LINES					    */
        LFC_CONTOUR_SUPP     = 39,  /* SUPPLEMENTARY CONTOUR LINES              */
        LFC_MAX_CONTOUR      = 39,

        // Waterways
        LFC_MIN_WATER        = 40,
        LFC_RIVER_UNKNOWN    = 40,  /* STREAM OR RIVER (UNKNOWN LENGTH)         */
        LFC_RIVER_LT_20KM    = 41,  /* STREAM OR RIVER (< 12KM )                */
        LFC_RIVER_50KM       = 42,  /* STREAM OR RIVER (12 - 50KM )             */
        LFC_RIVER_100KM      = 43,  /* STREAM OR RIVER (50 - 100KM )            */
        LFC_RIVER_250KM      = 44,  /* STREAM OR RIVER (100 - 250KM )           */
        LFC_RIVER_GT_250KM   = 45,  /* STREAM OR RIVER (> 250KM )               */
        LFC_MAX_WATER        = 45,

	    // Bathymetric contour lines
        LFC_MIN_BATHY_CONTOUR= 46,
	    LFC_BATHY_CONTOUR    = 46,  /* BATHYMETRIC CONTOUR LINE					*/
	    LFC_BATHY_MINOR		 = 47,  /* MINOR BATHYMETRIC CONTOUR LINE		    */
	    LFC_BATHY_MAJOR      = 48,  /* MAJOR BATHYMETRIC CONTOUR LINE           */
        LFC_BATHY_SUPP       = 49,  /* SUPPLEMENTARY BATHYMETRIC CONTOUR LINE   */
        LFC_MAX_BATHY_CONTOUR= 49,

        LFC_UNKNOWN          = 50,

        // Other line types
        LFC_MEASUREMENT      = 51,      // Measurement
        LFC_RUNWAY           = 52,      // Airport runway
        LFC_TRACKLOG         = 53,      // GPS Tracklog
        LFC_MARINE_ROUTE     = 54,      // Marine (recommended) route
        LFC_DIGITIZER_LINE   = 55,      // Active draw line in Digitizer Tool
        LFC_DELETED_LINE     = 56,      // Deleted line (used for style customization of deleted line features)
        LFC_SELECTED_LINE    = 57,      // Selected line (used for style customization of selected line features)
        LFC_MAX_LINE         = 57,

        LFC_FIRST_CUSTOM     = 0x200,   // First custom line type

        LFC_INVALID          = 0xFFFF
    };

    /* ----- Point feature classifications ----- */
    public enum PointFeatureClass_t16 : ushort
    {
        PFC_MIN_POINT      = 0,
        PFC_NONE           = 0,

        // City features
        PFC_MIN_CITY         =  1,
        PFC_CITY_10M         =  1,
        PFC_CITY_1M          =  4,
        PFC_CITY_500K        =  5,
        PFC_CITY_100K        =  7,
        PFC_CITY_50K         =  8,
        PFC_CITY_10K         = 10,
        PFC_CITY_LT10K       = 12,
        PFC_CITY_UNKNOWN     = 13,  /* CITY CENTER, POPULATION UNKNOWN      */
        PFC_MAX_CITY         = 13,

        // Political entities
        PFC_MIN_POLITICAL    = 20,
        PFC_COUNTRY          = 20,  /* COUNTRY                              */
        PFC_STATE            = 22,  /* STATE, PROVINCE, ECT.                */
        PFC_COUNTY           = 23,  /* COUNTY, PARISH, ECT.                 */
        PFC_ISLAND           = 24,  /* ISLAND (ANY SIZE)                    */
        PFC_LAND_GRANT       = 25,
        PFC_MAX_POLITICAL    = 25,

        // Marine point feature
        PFC_MIN_MARINE       = 30,
        PFC_ANCHORAGE        = 30,  /* ANCHORAGE                            */
        PFC_BEACON           = 31,  /* BEACON                               */
        PFC_BOTTOM_CONDITIONS= 32,  /* BOTTOM CONDITIONS                    */
        PFC_BUOY             = 33,  /* BUOY                                 */
        PFC_DANGER           = 34,  /* DANGER POINT                         */
        PFC_LIGHTHOUSE       = 35,  /* LIGHTHOUSE                           */
        PFC_SPOT_SOUNDING    = 36,  /* SPOT SOUNDING                        */
        PFC_MAX_MARINE       = 39,

        // Lidar point types
        PFC_MIN_LIDAR = 40,
        PFC_LIDAR_UNCLASSIFIED      = 40,   /* LIDAR - UNCLASSIFIED POINT   */
        PFC_LIDAR_BUILDING          = 41,   /* LIDAR - BUILDING             */
        PFC_LIDAR_GROUND            = 42,   /* LIDAR - GROUND SHOT POINT    */
        PFC_LIDAR_VEGETATION_LOW    = 43,   /* LIDAR - LOW VEGETATION       */
        PFC_LIDAR_VEGETATION_MEDIUM = 44,   /* LIDAR - MEDIUM VEGETATION    */
        PFC_LIDAR_VEGETATION_HIGH   = 45,   /* LIDAR - HIGH VEGETATION      */
        PFC_LIDAR_WATER             = 46,   /* LIDAR - WATER                */
        PFC_LIDAR_POWER_LINES       = 47,   /* LIDAR - POWER LINES          */
        PFC_LIDAR_TOWERS_OR_POLES   = 48,   /* LIDAR - TOWERS OR POLES      */
        PFC_MAX_LIDAR = 48,

        // Misc point features
        PFC_STRIKE_AND_DIP   = 56,  /* STRIKE AND DIP (GEOLOGY DATA)        */
        PFC_PICTURE          = 57,  /* PICTURE LOCATION (i.e. EXIF DATA)    */
        PFC_ADDRESS_LABEL    = 58,  /* LABEL USED ON DOWNLOADED LAYERS      */
        PFC_WAYPOINT         = 59,  /* USER WAYPOINT                        */
        PFC_ARCH             = 60,  /* ARCH                                 */
        PFC_BRIDGE           = 61,  /* BRIDGE                               */
        PFC_CLIFF            = 62,  /* CLIFF                                */
        PFC_WATERFALL        = 63,  /* WATERFALL                            */
        PFC_GEYSER           = 64,  /* GEYSER                               */
        PFC_SUMMIT           = 65,  /* SUMMIT                               */
        PFC_WRECK            = 66,  /* WRECK                                */
        PFC_MARINA           = 67,  /* MARINA                               */
        PFC_WELL             = 68,  /* WELL                                 */
        PFC_TOWER            = 69,  /* TOWER                                */
        PFC_ROCK             = 70,  /* ROCK                                 */
        PFC_BOAT_RAMP        = 71,  /* BOAT RAMP                            */
        PFC_CAMPGROUND       = 72,  /* CAMPGROUND                           */
        PFC_PARK             = 73,  /* PARK                                 */
        PFC_PICNIC_AREA      = 74,  /* PICNIC AREA                          */
        PFC_1ST_AID          = 75,  /* FIRST AID STATION                    */
        PFC_MILITARY         = 76,  /* MILITARY INSTALLATION OR RESERVATION */
        PFC_LANDMARK         = 77,  /* GENERIC LANDMARK                     */
        PFC_RESTROOMS        = 78,  /* RESTROOMS                            */
        PFC_LODGING          = 79,  /* HOTEL/MOTEL/ETC.                     */
        PFC_SHOPPING         = 80,  /* SHOPPING                             */
        PFC_GOLF_COURSE      = 81,  /* GOLF COURSE                          */
        PFC_LIBRARY          = 82,  /* LIBRARY                              */
        PFC_HOSPITAL         = 83,  /* HOSPITAL                             */
        PFC_SWIMMING_AREA    = 84,  /* SWIMMING AREA                        */
        PFC_DAM              = 85,  /* DAM                                  */
        PFC_MINE             = 86,  /* MINE                                 */
        PFC_BUILDING         = 87,  /* BUILDING                             */
        PFC_BANK             = 88,  /* BANK                                 */
        PFC_AIRPORT          = 89,  /* AIRPORT                              */
        PFC_SCHOOL           = 90,  /* SCHOOL                               */
        PFC_CHURCH           = 91,  /* CHURCH                               */
        PFC_POST_OFFICE      = 92,  /* POST OFFICE                          */
        PFC_CEMETERY         = 93,  /* CEMETERY                             */
        PFC_RESTAURANT       = 94,  /* RESTAURANT/DINING                    */
        PFC_WEIGH_STATION    = 95,  /* WEIGH STATION                        */
        PFC_AMUSEMENT        = 96,  /* AMUSEMENT PARK, AREA                 */
        PFC_STADIUM          = 97,  /* STADIUM                              */
        PFC_SPOT_ELEVATION   = 98,  /* SPOT ELEVATION                       */
        PFC_STREAM_ORIGIN    = 99,  /* STREAM ORIGIN                        */

        PFC_UNKNOWN          = 100,
        PFC_MAX_POINT        = 100,

        PFC_FIRST_CUSTOM     = 0x200,   // First custom point type

        PFC_INVALID          = 0xFFFF
    };

    // List of supported elevation units
    public enum GM_ElevUnits_t8 : byte
    {
        GM_ElevUnit_Meters = 0,
        GM_ElevUnit_Decimeters = 1,
        GM_ElevUnit_Centimeters = 2,
        GM_ElevUnit_Millimeters = 3,
        GM_ElevUnit_Feet = 4,
        GM_ElevUnit_Decifeet = 5,
        GM_ElevUnit_Inches = 6,
        GM_ElevUnit_Centifeet = 7,

        GM_ElevUnit_NumUnits
    };

	// This type is used when rectifying imagery
	public struct GM_GroundControlPoint_t
	{
		public double	PixelX;    // x pixel coordinate of GCP
		public double	PixelY;    // y pixel coordinate of GCP (top is 0, increases down)
		public double	GroundX;   // x ground coordinate of GCP (in provided projection)
		public double	GroundY;   // y ground coordinate of GCP (in provided projection)
	};

	// This type is used when returning information about a layer
	public struct GM_LayerInfo_t
	{
		public string						mDescription;   	// Description string
		public GM_Rectangle_t				mNativeRect;    	// Bounding rect of layer in native coordinates
		public GM_Rectangle_t				mGlobalRect;    	// Bounding rect of layer in global coordinates
		public UInt32						mPixelWidth;    	// RASTER/ELEV ONLY: Number of pixels wide layer is
		public UInt32						mPixelHeight;   	// RASTER/ELEV ONLY: Number of pixels tall layer is
		public GM_Projection_t				mNativeProj;    	// Native projection of layer
		public IntPtr						mControlPoints; 	// RASTER ONLY: Ground control points list.  Point to GM_GroundControlPoint_t
		public UInt32						mNumGCPs;       	// RASTER ONLY: Number of ground control points in list
		public float						mMinElevation;  	// ELEV ONLY: Minimum elevation in meters
		public float						mMaxElevation;  	// ELEV ONLY: Maximum elevation in meters
		public UInt32						mNumAreas;      	// VECTOR ONLY: Number of area features
		public UInt32						mNumLines;      	// VECTOR ONLY: Number of line features
		public UInt32						mNumPoints;     	// VECTOR ONLY: Number of point features
		public double                       mPixelSizeX;		// RASTER/ELEV ONLY: Pixel size in meters in the x direction
		public double                       mPixelSizeY;        // RASTER/ELEV ONLY: Pixel size in meters in the y direction
		public Byte 	                    mHasRasterData;     // Does this layer have raster or elevation data?
		public Byte				  			mEnabled;           // Is this layer enabled for display or it it hidden?
		public Byte                         mHasVectorData;     // Does this layer have vector data (the features counts can all be 0 for things like map catalogs)
		public Byte                         mUsedDefaultPos;    // RASTER/ELEV ONLY: Was the default position used for this layer since the file could not be automatically positioned?
		public string                       mFilename;          // Filename from which layer was loaded (if this is an archive file, like .zip, then the name of the actual loaded file will be in mArchiveFilename
		public string                       mArchiveFilename;   // Filename within archive file (e.g. .zip or .tar.gz) from which layer was loaded, if any (might be NULL)
		public string                       mTypeName;          // Layer type name
		public UInt32                       mGlobalPixelWidth;  // RASTER/ELEV ONLY: Approximate number of pixels required in width for a 1:1 pixel mapping in the current projection
		public UInt32                       mGlobalPixelHeight; // RASTER/ELEV ONLY: Approximate number of pixels required in height for a 1:1 pixel mapping in the current projection
		public IntPtr						mMetadataList;      // List of metadata attributes and values for the layer
		public UInt32                       mMetadataListSize;  // Number of entries in mMetadataList
        public UInt32                       mNumBands;          // Number of bands in a raster image (use GM_SetRasterDisplayOptions to change color band layout)
        public string                       mExtraLoadFlags;    // Additional load flags with options selected by user during load. Pass these to GM_LoadLayerListEx to remember options.
    };

    // This type is used when returning a color
    public struct GM_Color_t
    {
        public Byte mRed;       // Red component of color
        public Byte mGreen;     // Green component of color
        public Byte mBlue;      // Blue component of color
    };
    
    // NOTE: The brush styles are stored in the registry so only add new
    //       syles at the end.
    public enum GM_BrushStyle_t16 : ushort
    {
        GM_BRUSH_NULL,
        GM_BRUSH_SOLID,

        // Hatched brushes
        GM_BRUSH_MIN_HATCH,
        GM_BRUSH_BACK_DIAG_CROSSHATCH = GM_BRUSH_MIN_HATCH,
        GM_BRUSH_CROSSHATCH,
        GM_BRUSH_DIAG_CROSSHATCH,
        GM_BRUSH_FWD_DIAG_CROSSHATCH,
        GM_BRUSH_HORZ_HATCH,
        GM_BRUSH_VERT_HATCH,
        GM_BRUSH_MAX_HATCH = GM_BRUSH_VERT_HATCH,

        // Pattern brushes
        GM_BRUSH_MIN_PATTERN,
        GM_BRUSH_SAND_PATTERN = GM_BRUSH_MIN_PATTERN,
        GM_BRUSH_INT_WATER_PATTERN,
        GM_BRUSH_SALT_PATTERN,
        GM_BRUSH_CORAL_PATTERN,
        GM_BRUSH_BLACK_CROSS_PATTERN,
        GM_BRUSH_BLACK_DOT_PATTERN,
        GM_BRUSH_MAX_PATTERN = GM_BRUSH_BLACK_DOT_PATTERN,

        GM_BRUSH_NUM_STYLES,

        // Custom Symbol support
        GM_BRUSH_FIRST_CUSTOM = 0x100
    };

    // NOTE: The pen styles are stored in the registry so only add new
    //       syles at the end.
    public enum GM_PenStyle_t16 : ushort
    { 
        GM_PEN_SOLID = 0,
        GM_PEN_DASH,
        GM_PEN_DOT,
        GM_PEN_DASHDOT,
        GM_PEN_DASHDOTDOT,
        GM_PEN_NULL,
            
        // Stylized lines
        GM_PEN_RAILROAD,
        GM_PEN_STRIPED,                 // A solid pen with a single pixel black stripe down the middle

        // Comb/rake/etc. stylized lines
        GM_PEN_FIRST_DIRECTED,
        GM_PEN_LEFT_FACING_LINE = GM_PEN_FIRST_DIRECTED,    // A railroad tie facing to the left of a line
        GM_PEN_RIGHT_FACING_LINE,                           // A railroad tie facing to the right of a line
        GM_PEN_COMB_LONG_TEETH_LEFT,                        // A long railroad tie facing to the left of a line
        GM_PEN_COMB_LONG_TEETH_RIGHT,                       // A long railroad tie facing to the right of a line
        GM_PEN_COMB_WIDE_SPACED_LEFT,                       // Widely spaced railroad ties facing to the left of a line
        GM_PEN_COMB_WIDE_SPACED_RIGHT,                      // Widely spaced railroad ties facing to the right of a line
        GM_PEN_COMB_ALTERNATING_LEFT,                       // Alternating long and short railroad ties to the left of a line
        GM_PEN_COMB_ALTERNATING_RIGHT,                      // Alternating long and short railroad ties to the right of a line
        GM_PEN_COMB_COUPLED_LEFT,                           // Double teeth to the left of a line
        GM_PEN_COMB_COUPLED_RIGHT,                          // Double teeth to the right of a line
        GM_PEN_COMB_TRIPLET_LEFT,                           // Triplet of teeth to the left of a line
        GM_PEN_COMB_TRIPLET_RIGHT,                          // Triplet teeth to the right of a line
        GM_PEN_RAKE_NARROW_SPACED_LEFT,                     // Narrowly space rake of triangles to the left of a line
        GM_PEN_RAKE_NARROW_SPACED_RIGHT,                    // Narrowly space rake of triangles to the right of a line
        GM_PEN_RAKE_LARGE_SPACED_LEFT,                      // Large spaced rake of triangles to the left of a line
        GM_PEN_RAKE_LARGE_SPACED_RIGHT,                     // Large spaced rake of triangles to the right of a line
        GM_PEN_MINUS_SIGNS_LEFT,                            // Minus signs to left of a line
        GM_PEN_MINUS_SIGNS_RIGHT,                           // Minus signs to right of a line
        GM_PEN_PLUS_SIGNS_LEFT,                             // Plus signs to left of a line
        GM_PEN_PLUS_SIGNS_RIGHT,                            // Plus signs to right of a line
        GM_PEN_DOTS_LEFT,                                   // Dots to left of a line
        GM_PEN_DOTS_RIGHT,                                  // Dots to right of a line
        GM_PEN_T_SHAPED_LEFT,                               // T-shape to left of a line
        GM_PEN_T_SHAPED_RIGHT,                              // T-shape to right of a line
        GM_PEN_RECTS_LEFT,                                  // Rectangles to left of a line
        GM_PEN_RECTS_RIGHT,                                 // Rectangles to right of a line
        GM_PEN_SQUARES_LEFT,                                // Squares to left of a line
        GM_PEN_SQUARES_RIGHT,                               // Squares to right of a line
        GM_PEN_HALF_CIRCLE_NARROW_LEFT,                     // Narrow-spaced half circles to the left of a line
        GM_PEN_HALF_CIRCLE_NARROW_RIGHT,                    // Narrow-spaced half circles to the right of a line
        GM_PEN_HALF_CIRCLE_WIDE_LEFT,                       // Widely-spaced half circles to the left of a line
        GM_PEN_HALF_CIRCLE_WIDE_RIGHT,                      // Widely-spaced half circles to the right of a line
        GM_PEN_DIAMONDS_LEFT,                               // Filled diamonds to the left of a line
        GM_PEN_DIAMONDS_RIGHT,                              // Filled diamonds to the right of a line
        GM_PEN_LAST_DIRECTED = GM_PEN_DIAMONDS_RIGHT,

        // Special symbol styles that are directed track of symbols towards one end of the line or the other
        GM_PEN_FIRST_DIRECTED_TRACK = 0x40,
        GM_PEN_V_TRACK_FORWARD = GM_PEN_FIRST_DIRECTED_TRACK,   // Line made up of V symbols opening away the start of the line
        GM_PEN_V_TRACK_REVERSE,                                 // Line made up of V symbols opening towards the start of the line
        GM_PEN_C_TRACK_FORWARD,                                 // Line made up of C symbols opening away from the start of the line
        GM_PEN_C_TRACK_REVERSE,                                 // Line made up of C symbols opening towards from the start of the line
        GM_PEN_ARROW_TRACK_FORWARD,                             // Line made up of triangles pointing away the start of the line
        GM_PEN_ARROW_TRACK_REVERSE,                             // Line made up of triangles pointing towards the start of the line
        GM_PEN_V_TRACK_SOLID_FORWARD,                           // Line made up of V symbols opening away the start of the solid line
        GM_PEN_V_TRACK_SOLID_REVERSE,                           // Line made up of V symbols opening towards the start of the solid line
        GM_PEN_LAST_DIRECTED_TRACK = GM_PEN_V_TRACK_SOLID_REVERSE,

        // Special symbol styles along solid line
        GM_PEN_FIRST_SOLID_SYMBOL = 0x80,
        GM_PEN_SOLID_WITH_X = GM_PEN_FIRST_SOLID_SYMBOL,// Solid line with X's through them
        GM_PEN_SOLID_DIAMONDS,                          // Solid line with solid diamonds through them
        GM_PEN_SOLID_EMPTY_DIAMONDS,                    // Solid line with empty diamonds through them
        GM_PEN_SOLID_CIRCLES,                           // Solid line with solid circles through them
        GM_PEN_SOLID_EMPTY_CIRCLES,                     // Solid line with empty circles through them
        GM_PEN_LAST_SOLID_SYMBOL = GM_PEN_SOLID_EMPTY_CIRCLES,

        // Styles with special symbols at end
        GM_PEN_FIRST_END_SYMBOL = 0x100,
        GM_PEN_ARROW_TO_START = GM_PEN_FIRST_END_SYMBOL,    // Solid line with an arrow pointing to the start of the line
        GM_PEN_ARROW_TO_END,                                // Solid line with an arrow pointing to the end of the line
        GM_PEN_EMPTY_CIRCLE_AT_START,                       // Solid line with an empty circle at the start of the line
        GM_PEN_EMPTY_CIRCLE_AT_END,                         // Solid line with an empty circle at the end of the line
        GM_PEN_LAST_END_SYMBOL = GM_PEN_EMPTY_CIRCLE_AT_END,

        // Additional styles
        GM_PEN_STRIPED_BLACK_EDGES,                     // A solid  pen with a single pixel black stripe down each edge

        GM_PEN_NUM_STYLES,
        GM_PEN_FIRST_STYLIZED = GM_PEN_RAILROAD,
    };

    // NOTE: The orientations are stored in the registry so only add new
    //       values at the end.
    public enum GM_LabelPlacement_t8 : byte
    {
        GM_LABEL_AUTO = 0,                  // automatically place label at best location
        GM_LABEL_CENTER = 1,                // center the label on the point
        GM_LABEL_TOP_RIGHT = 2,             // label is up and to the right of point
        GM_LABEL_RIGHT = 3,                 // label is to the right of the point and vertically centered on it
        GM_LABEL_BOTTOM_RIGHT = 4,          // label is below and to the right of point
        GM_LABEL_BOTTOM = 5,                // label is below the point and horizontally centered on it
        GM_LABEL_BOTTOM_LEFT = 6,           // label is below and to the left of point
        GM_LABEL_LEFT = 7,                  // label is left of the point and vertically centered on it
        GM_LABEL_TOP_LEFT = 8,              // label is above and to the left of point
        GM_LABEL_TOP = 9,                   // label is above the point and horizontally centered on it

        GM_LABEL_NUM_PLACEMENTS
    };

    // This type represents the different font background options
	public enum GM_FontBackground_t8 : byte
    {
        GM_Font_BgAuto          = 0,    // Automatically choose transparent or opaque background (all transparent except when centered on line)
        GM_Font_BgOpaque        = 1,    // Fill label background with current background color
        GM_Font_BgTransparent   = 2,    // Make label background transparent
    };

    // This type is used to describe a font used for rendering text
    public struct GM_FontDef_t
    {
        public IntPtr               mFaceName;  // name of font face used (char*, because it's not a const char* we can't just use the C# 'string' type)
        public UInt32               mColor;     // RGB color of font
        public UInt16               mWeight;    // weight of font (use Win32 constants, like FW_BOLD, FW_THIN, FW_NORMAL, etc.)
        public Byte                 mPointSize; // font point size (set mFixedHgt value to non-zero to use fixed height)
        public Byte                 mItalicize; // draw text with italics?
        public Byte                 mUnderline; // underline text?
        public Byte                 mStrikeout; // strikeout text?
        public Byte                 mCharset;   // character set (use OEM_CHARSET for default behavior)
        public GM_FontBackground_t8 mBgMode;    // background fill mode (i.e. opaque or transparent)
        public float                mFixedHgt;  // fixed height of font in meters (use 0.0 for normal point-size font)
        public GM_LabelPlacement_t8 mPlacement; // label placement relative to point features (only applies for point features)
        public Byte                 mHalo;      // draw thin halo around text to make it visible on any background.
        public Byte                 mReserved2; // reserved, must be 0
        public Byte                 mReserved3; // reserved, must be 0
        public float                mAngle;     // counter-clockwise rotation angle of angle (0 is horizontal left to right)
    };

    // This type is used to describe a style used for drawing an area feature
    // and its label (if any)
    public struct GM_AreaStyle_t
    {
        public GM_PenStyle_t16      mBorderPenStyle;    // border pen style (i.e. solid, dash, etc.)
        public UInt16               mBorderPenWidth;    // width to draw border pen in
        public UInt32               mBorderPenColor;    // color to use for border pen
        public UInt32               mBrushColor;        // color of brush to fill area with (when applicable to brush style). The color is defined as ARGB where the upper 8 bits are alpha. The exception is a value of 0 in the A is treated as no alpha.
        public GM_BrushStyle_t16    mBrushStyle;        // style of brush to fill area with
        public Byte                 mDrawLabel;         // render the label for this area if there is one
        public Byte                 mDrawLabelAlways;   // render the label for this area even if it collides with another display label
        public GM_FontDef_t         mFont;              // font to use to render label
    };

    // This type is used to describe a style used for drawing a line feature
    // and its label (if any)
    public struct GM_LineStyle_t
    {
        public GM_PenStyle_t16  mPenStyle;          // pen style (i.e. solid, dash, etc.)
        public UInt16           mPenWidth;          // width in pixels to draw pen in (set to 0 for fixed width lines)
        public UInt32           mPenColor;          // color to use for pen
        public Byte             mDrawLabel;         // render the label for this line if there is one
        public Byte             mDrawLabelAlways;   // render the label for this line even if it collides with another display label
        public Byte             mDrawLabelOnLine;   // render the label for this line (if any) directly on the line rather than offset from it
        public Byte             mReserved;          // must be 0
        public GM_FontDef_t     mFont;              // font to use to render label
        public Int32            mLabelSegment;      // segment number (1-based) on which to render label (use 0 to use default label placement)
        public float            mFixedWidthMeters;  // fixed width in meters to use for pen (only used if mPenWidth is zero)
    };

    // This type is OR'd with the mRotation member of the GM_PointStyle_t structure to specify
    // symbol rotation sense.
    public enum GM_RotationSense_t16 : ushort
    {
        GM_RotateSense_TopOfMap  = 0x0000,  // Rotation is to top of map (default)
        GM_RotateSense_TrueNorth = 0x1000,  // Rotation is relative to true north
        GM_RotateSense_MagNorth  = 0x2000,  // Rotation is relative to magnetic north

        GM_RotateSense_Mask      = 0xF000,
        GM_RotateAngle_Mask      = 0x0FFF
    };

    // This type is used to describe a style used for drawing a point feature
    // and its label (if any)
    public struct GM_PointStyle_t
    {
        public string       mSymbolName;        // name of symbol
        public GM_FontDef_t mFont;              // font to use to render label
        public Byte         mDrawLabel;         // render the label for this point if there is one
        public Byte         mDrawLabelAlways;   // render the label for this point even if it collides with another display label
        public UInt16       mRotation;          // rotation angle in degrees (0 is up/north, 90 right/east, etc.) and rotation sense flags
        public float        mScale;             // symbol scaling factor to apply (use negative values to make symbol a fixed height in meters, i.e. -10 would make symbol 10 meters high)
    };

    // This type is used to represent a single attribute value pair
    unsafe public struct GM_AttrValue_t
    {
        public char*    mName;      // Name of the attribute
        public char*    mVal;       // Value of the attribute
    };

    // This type is used as the base for any vector feature information
    public struct GM_VectorFeature_t
    {
        public string       mName;      // Name of the feature
        public string       mDesc;      // Description of the feature
        public UInt16       mClass;     // Global Mapper classification assigned to the feature
        public IntPtr       mAttrList;  // List of attributes associated with feature (pointer to GM_AttrValue_t array)
        public UInt16       mNumAttrs;  // Number of attributes in mAttrList
    };

    // This type is used to store information about holes within area features
    public struct GM_HoleInArea_t
    {
        public IntPtr   mPoints;    // List of points in the hole in global coordinates (pointer to GM_Point_t array)
        public UInt32   mNumPoints; // Number of points in the list
    };

    // This type is used to return information about area features
    public struct GM_AreaFeature_t
    {
        public GM_VectorFeature_t   mFeatureInfo;   // General feature info
        public IntPtr               mPointList;     // List of points in the area (global coordinates when getting, native when adding feature) (pointer to GM_Point_t array)
        public UInt32               mNumPoints;     // Number of points in the list
        public IntPtr               mHoleList;      // List of holes in the area (pointer to GM_HoleInArea_t array)
        public UInt32               mNumHoles;      // Number of holes in the list of holes
        public GM_AreaStyle_t       mAreaStyle;     // Area render style
        public IntPtr               mVertexElevList;// Optional list of elevations for each point in area. If present, must have <mNumPoints> values (pointer to float array)
    };

    // This type is used to return information about line features
    public struct GM_LineFeature_t
    {
        public GM_VectorFeature_t   mFeatureInfo;   // General feature info
        public IntPtr               mPointList;     // List of points in the line (global coordinates when getting, native when adding feature) (pointer to GM_Point_t array)
        public UInt32               mNumPoints;     // Number of points in the list
        public GM_LineStyle_t       mLineStyle;     // Line render style
        public IntPtr               mVertexElevList;// Optional list of elevations for each point in line. If present, must have <mNumPoints> values (pointer to float array)
    };

    // This type is used to return information about point features
    public struct GM_PointFeature_t
    {
        public GM_VectorFeature_t   mFeatureInfo;   // General feature info
        public GM_Point_t           mPos;           // Location of the point (global coordinates when getting, native when adding feature)
        public GM_PointStyle_t      mPointStyle;    // Point render style (symbol and font)
    };

	/// <summary>
	/// Summary description for GlobalMapper.
	/// </summary>
	public class GlobalMapperDLL
	{
		// Define the filename for the DLL file to use
		private const string DLLFileName = @"..\GlobalMapperInterface.dll";

		// Error codes
		public enum GM_Error_t32 : uint
		{
			GM_Error_None = 0,					// The function completed successfully
			GM_Error_InvalidLayer = 1,          // The provided layer is not open
			GM_Error_InvalidParam = 2,          // A parameter was invalid
			GM_Error_LoadError = 3,             // Unable to load layer
			GM_Error_MultipleLayers = 4,        // Multiple layers were found in the file being loaded (call GM_LoadLayerList instead)
			GM_Error_CollarSetup = 5,           // Unable to automatically crop the collar
			GM_Error_DrawError = 6,             // An unknown error occured drawing the layer(s)
			GM_Error_ExportError = 7,           // An unknown error occured exporting the layer(s)
			GM_Error_NothingToExport = 8,       // No data was available to export
			GM_Error_InvalidFormat = 9,         // An invalid export format was specified
			GM_Error_RectifyError = 10,         // An error occured rectifying the image
			GM_Error_NotSupported = 11,         // The operation is not supported for the layer
			GM_Error_NotRegistered = 12,        // No registry key was found to enable this functionality
			GM_Error_NoDataAtLocation = 13,     // No data was found at the specified location
			GM_Error_OutOfMemory = 14,          // Not enough memory to complete the requested operation
			GM_Error_Projection = 15,           // An error occured projecting a coordinate
			GM_Error_NoProjectionSet = 16,      // No view/export projection is currently set
			GM_Error_ViewShedCalc = 17,         // An error occurred performing the view shed analysis
			GM_Error_OperationCanceled = 18,    // The user canceled the operation
			GM_Error_TerraServerConnect = 19,   // A connection to the TerraService could not be established
			GM_Error_TerraServerBadTheme = 20,  // No TerraServer theme with the theme name provided could be found
			GM_Error_GenerateContours = 21,     // An error occurred generating the contours
			GM_Error_NotRasterLayer = 22,       // The layer provided is not a raster or elevation layer
			GM_Error_InvalidPackage = 23,       // The provided handle is not a valid package
			GM_Error_InvalidFeatureIndex = 24,  // There was no feature at the given index in the layer
			GM_Error_InvalidDrawStyle = 25,     // The drawing style passed in is not valid
			GM_Error_InvalidSymbolName = 26,    // The symbol name provided does not match any available point symbols
			GM_Error_SymbolAlreadyCreated = 27, // The symbol name specified has already been added
			GM_Error_MissingFile = 28,          // The file specified could not be found
			GM_Error_GPSDataNotValid = 29,      // The GPS data requested does not currently have a valid value available
			GM_Error_GPSAlreadyTracking = 30,   // A GPS device is already being tracked, call GM_GPSStopTracking
			GM_Error_VolumeCalcFailed = 31,     // An error occurred calculating the volume
			GM_Error_InvalidHoleIndex = 32,     // The area does not have a hole at the index provided
			GM_Error_InvalidMapIndex = 33,      // There is no map at the given index in the map catalog
			GM_Error_3DNotOpen = 34,            // No 3D view window is currently open
			GM_Error_3DLibraryNotFound = 35,    // The ExternalViewerIntf.dll library was not found
			GM_Error_3DLibraryBadInterface = 36,// The ExternalViewerIntf.dll library did not have the proper interface
			GM_Error_3DCreateError = 37,        // Unexpected error creating 3D view
			GM_Error_3DSetViewError = 38,       // Unexpected error setting 3D view
			GM_Error_NothingFound = 39,         // No features were found within the search area
			GM_Error_SearchError = 40,          // Unexpected search area
			GM_Error_LowMemory = 41,            // Some data was not drawn due to low available memory
			GM_Error_NotElevationLayer = 42,    // The layer provided is not an elevation layer
			GM_Error_FileAlreadyInCatalog = 43, // The provided file was already in the map catalog to which it was being added.
			GM_Error_TypeAlreadyExists = 44,    // The provided type name was already used by an existing type
			GM_Error_DatumAlreadyExists = 45,   // A built-in datum with the given name already exists
			GM_Error_UnknownEllipsoid = 46,     // The ellipsoid name provided does not match a known ellipsoid
            GM_Error_GridError = 47,            // Error generating grid from 3D vector data
            GM_Error_InvalidShaderName = 48,    // The shader name provided was not found

			GM_Error_NotImplemented = 0xFFFF    // The given function hasn't been implemented yet
		};

		// Flags when loading new layres
		public enum GM_LoadFlags_t32 : uint
		{
			GM_LoadFlags_HideProgress       = 0x00000001,   // Don't show load progress
			GM_LoadFlags_NoProjPrompt       = 0x00000002,   // Don't prompt the user for unknown projections
			GM_LoadFlags_UseDefaultProj     = 0x00000004,   // Use default projection if none provided
			GM_LoadFlags_HideWarnings       = 0x00000008,   // Hide any load warnings (load errors will still be displayed)
			GM_LoadFlags_HideUnknownPrompt  = 0x00000010,   // Do not prompt the user to select the file type for unknown file formats
			GM_LoadFlags_UseDefaultPos      = 0x00000020,   // Use default positioning if none provided (raster files only)
			GM_LoadFlags_HideAllPrompts     = 0x00000040,   // Use the default value for any prompts that display during the load
		};

        // Flags to control the GM_DrawLayerList and GM_DrawPackageList operations
        public enum GM_DrawFlags_t32 : uint
        {
            GM_DrawFlags_BestRasterLayers       = 0x00000001,   // Only draw the best raster layers for this zoom scale
            GM_DrawFlags_BestRasterLayersHiRes  = 0x00000003,   // Same as GM_DrawFlags_BestRasterLayers, only it shows higher resolution layers a bit sooner
            GM_DrawFlags_LowResRasterLayers     = 0x00000002,   // Only draw the lowest resolution raster layers
            GM_DrawFlags_HiResRasterLayers      = 0x00000004,   // Only draw the highest resolution raster layers
            GM_DrawFlags_BestRasterChart        = 0x00000005,   // Only draw the best raster chart (this is tuned for marine chart display)

            GM_DrawFlags_EraseBackground        = 0x00000100,   // Erase the background with the current background color (GM_SetBackgroundColor)
            GM_DrawFlags_HideAreas              = 0x00000200,   // Do not draw any area features from vector layers
            GM_DrawFlags_HideLines              = 0x00000400,   // Do not draw any line features from vector layers
            GM_DrawFlags_HidePoints             = 0x00000800,   // Do not draw any point features from vector layers
            GM_DrawFlags_SeparateVectorLayers   = 0x00001000,   // Draw each vector separately rather than drawing all at once.
            GM_DrawFlags_HideLabels             = 0x00002000,   // Do not label any rendered features
            GM_DrawFlags_DontMaintainAspectRatio= 0x00004000,   // Do NOT maintain the aspect ratio when drawing (this allows stretched views)
            GM_DrawFlags_HidePartialLabels      = 0x00008000,   // Hide labels that are not entirely in the draw window
            GM_DrawFlags_ShowRenderProgress     = 0x00010000,   // Show render progress dialog or call progress callback if provided
            GM_DrawFlags_HideExtraIconText      = 0x00020000,   // Hide any extra text shown below road icons
            GM_DrawFlags_HideNonIconRoadLabels  = 0x00040000,   // Hide any road line labels that are not iconized
            GM_DrawFlags_RenderVertices         = 0x00080000,   // Render vertices for area and line features
            GM_DrawFlags_IncrementalRender      = 0x00100000,   // Do an incremental render so that draw progress can be seen as layers complete rendering
            GM_DrawFlags_NoAutoInterpolation    = 0x00200000,   // Disable automatic interpolation of rasters at intermediate zoom ranges (results in faster renders w/ less quality)

            // Definitions for masks of various option bits
            GM_DrawFlags_RasterMask             = 0x00000007,   // Mask for raster draw options
        };
        // This type represents the different contrast adjustment modes that are available
        public enum GM_ContrastMode_t8 : byte
        {
            GM_Contrast_None = 0,
            GM_Contrast_PercentageStretch,  // Stretch within some std-dev of mean to 0-255 range
            GM_Contrast_RangeStretch,       // Stretch entire valid range to 0-255 range

            GM_Contrast_NumModes,
            GM_Contrast_DefaultAuto = GM_Contrast_PercentageStretch
        };

        // This type represents the different blend modes available
        public enum GM_BlendMode_t8 : byte
        {
            GM_BlendMode_None = 0,
            GM_BlendMode_Multiple,
            GM_BlendMode_Screen,
            GM_BlendMode_Overlay,
            GM_BlendMode_HardLight,
            GM_BlendMode_ColorBurn,
            GM_BlendMode_ColorDodge,
            GM_BlendMode_ApplyColor,
            GM_BlendMode_ApplyColorReverse,
            GM_BlendMode_Darken,
            GM_BlendMode_Lighten,
            GM_BlendMode_Difference,
            GM_BlendMode_Exclusion,
            GM_BlendMode_KeepRedChannel,
            GM_BlendMode_KeepGreenChannel,
            GM_BlendMode_KeepBlueChannel,
            GM_BlendMode_SpotNaturalColor,      // view SPOT images as natural color
            GM_BlendMode_PsuedoNaturalColor,    // simple weighted average
            GM_BlendMode_ColorToGrayscale,      // convert RGB color image to grayscale

            // IMPORTANT NOTE: When adding new blend modes also add to the GM_BlendMode_t8 enumeration
            // in GlobalMapperInterface.h, which should match this.

            GM_BlendMode_NumModes
        };

        // Enumeration of different raster/elevation sampling methods available
        public enum GM_SamplingMethod_t8 : byte
        {
            GM_SAMPLING_NEAREST_NEIGHBOR,       // Nearest neighbor sampling
            GM_SAMPLING_BILINEAR_INTERPOLATION, // Bilinear interpolation
            GM_SAMPLING_BICUBIC_INTERPOLATION,  // Bicubic interpolation
            GM_SAMPLING_BOX_AVG_3X3,            // Box averager 3x3 grid
            GM_SAMPLING_BOX_AVG_4X4,            // Box averager 4x4 grid
            GM_SAMPLING_BOX_AVG_5X5,            // Box averager 5x5 grid
            GM_SAMPLING_BOX_AVG_7X7,            // Box averager 7x7 grid
            GM_SAMPLING_BOX_MAX_3X3,            // Box maximizer 3x3 grid
            GM_SAMPLING_BOX_MAX_4X4,            // Box maximizer 4x4 grid
            GM_SAMPLING_BOX_MAX_5X5,            // Box maximizer 5x5 grid
            GM_SAMPLING_BOX_MAX_7X7,            // Box maximizer 7x7 grid

            GM_NUM_SAMPLING_METHODS
        };

        // This type represents the display options available for raster and elevation layers
        public struct GM_RasterDisplayOptions_t
        {
            public UInt32               mSize;              // Size of structure
            public GM_ContrastMode_t8   mContrastMode;      // Contrast adjustment mode
            public byte                 mAutoClipCollar;    // Automatically crop off a DRG, BSB, or other known collar boundary
            public byte                 mColorIntensity;    // Color intensity (0-20). Lower values are lighter, higher values are darker.
            public GM_SamplingMethod_t8 mSamplingMethod;    // Specify how resampling should be done for this layer for display and export
            public byte                 mTextureMap;        // Texture map this raster layer over any underlying elevation layers.
            public GM_BlendMode_t8      mBlendMode;         // Blend mode to use to blend this raster layer with underlying layers or to interpret the color values of the layer

            // Translucent/transparency support
            public UInt16               mTranslucency;          // Translucency level of layer (0 - completely see-through, 512 - completely opaque)
            public byte                 mTransparent;           // Is layer transparent?
            public UInt32               mTransparentColor;      // Color to make transparent
            public byte                 mTransparentColorDist;  // Fuzzy transparency support. Distance from transparent color to treat nearby colors transparent.

            // Color channel adjustment
            public SByte                mRedAdjustPercent;      // Percentage to adjust red color channel by (-100 to 100)
            public SByte                mGreenAdjustPercent;    // Percentage to adjust red color channel by (-100 to 100)
            public SByte                mBlueAdjustPercent;     // Percentage to adjust red color channel by (-100 to 100)

            // Color band layout support
            public byte                 mBandLayoutValid;       // Are the band layout values valid?
            public byte                 mBandIdxRed;            // Index of color band (0-based) to use for red color channel
            public byte                 mBandIdxGreen;          // Index of color band (0-based) to use for green color channel
            public byte                 mBandIdxBlue;           // Index of color band (0-based) to use for blue color channel

            // Contrast adjustment parameters
            public double               mContrastNumStdDev;     // Number of standard deviations from mean to stretch for contrast adjustment (if 0 is used the default of 2 will be used)
            public byte                 mContrastShared;        // Is the contrast adjustment shared with other loaded layers?

            // Color Grade options
            public byte mColorGradeValid;       // Are the color grade options valid?
            public byte mColorGradeRedInMin;    // Minimum red input color value
            public byte mColorGradeRedInMax;    // Maximum red input color value
            public byte mColorGradeRedOutMin;   // Minimum red output color value
            public byte mColorGradeRedOutMax;   // Maximum red output color value
            public byte mColorGradeBlueInMin;   // Minimum Blue input color value
            public byte mColorGradeBlueInMax;   // Maximum Blue input color value
            public byte mColorGradeBlueOutMin;  // Minimum Blue output color value
            public byte mColorGradeBlueOutMax;  // Maximum Blue output color value
            public byte mColorGradeGreenInMin;  // Minimum Green input color value
            public byte mColorGradeGreenInMax;  // Maximum Green input color value
            public byte mColorGradeGreenOutMin; // Minimum Green output color value
            public byte mColorGradeGreenOutMax; // Maximum Green output color value
            public float mColorGradeSaturation; // Saturation (valid 0.0 - 1.0)

        };

		// List of supported raster export types
		public enum GM_RasterExportFormat_t32 : uint
		{
			GM_Export_GeoTIFF,          // GeoTIFF format
			GM_Export_JPG,              // JPG format
			GM_Export_PNG,              // PNG format
			GM_Export_Erdas_Imagine,    // Erdas Imagine IMG format
			GM_Export_BMP,              // BMP (Windows) format
			GM_Export_ECW,              // ECW format
            GM_Export_BIL_Raster,       // BIL format raster (RGB or grayscale)
            GM_Export_JPEG2000,         // JPEG 2000 format

			GM_Export_NumRasterFormats
		};

        public enum GM_WebExportFormat_t32 : uint
        {
            GM_Export_GoogleMaps,
            GM_Export_VirtualEarth,
            GM_Export_KMLRaster,
            GM_Export_NumWebFormats
        };

		// Flags for raster (i.e. GeoTIFF, JPG) image and elevation grid export
		public enum GM_RasterExportFlags_t32 : uint
		{
			GM_ExportFlags_GenWorldFile = 0x00000001,   // Generate a world file
			GM_ExportFlags_GenPRJFile   = 0x00000002,   // Generate a projection (PRJ) file
			GM_ExportFlags_GenTABFile   = 0x00000004,   // Generate a MapInfo TAB file
			GM_ExportFlags_Grayscale    = 0x00000008,   // Generate a grayscale image (raster only)
			GM_ExportFlags_Palette      = 0x00000010,   // Generate a paletted image (raster only)
			GM_ExportFlags_HideProgress = 0x00000020,   // Don't show export progress
			GM_ExportFlags_JPGinTIFF    = 0x00000040,   // Create a JPG compressed GeoTIFF file (GM_Export_GeoTIFF format only)
			GM_ExportFlags_FillSmallGaps= 0x00000080,   // Fill small gaps in data when exporting elevation data
			GM_ExportFlags_AllowLossy   = 0x00000100,   // Allow lossy export to GMG
			GM_ExportFlags_16BitSamples = 0x00000200,   // Use signed 16-bit integer elevation values (GM_Export_BIL and GM_Export_Erdas_Elevation formats only)
			GM_ExportFlags_UseLZW       = 0x00000400,   // Use LZW compression for GeoTIFF file (GM_Export_GeoTIFF format only)
			GM_ExportFlags_NoTransparent= 0x00000800,   // Don't mark background pixels transparent for PNG file (GM_Export_PNG format only)
            GM_ExportFlags_AddAlpha     = 0x00001000,   // Add an alpha channel to formats that support it (need for GM_Export_GeoTIFF [24-bit only], enabled by default for GM_Export_PNG)
        };

        // Flags for packageexport
        public enum GM_PackageExportFlags_t32 : uint
        {
            GM_ExportPackage_LatLongWGS84 = 0x00000001,         // Export data in lat/lon/WGS84 coordinates rather than the current projection or native projection of each layer
            GM_ExportPackage_DoublePrecision = 0x00000002,      // Use double-precision coordinates for vector data.
            GM_ExportPackage_DiscardAttributes = 0x00000004,    // Discard vector feature attributes 
            GM_ExportPackage_HideProgress = 0x00000008,         // Don't show export progress
            GM_ExportPackage_KeepAllStyles = 0x00000010,        // Keep all style information, even if features are using default style
            GM_ExportPackage_UseNativeProj = 0x00000020,        // Use the native projection for each layer rather than the current projection or lat/lon/WGS84
            GM_ExportPackage_CombineVectorLayers = 0x00000040,  // Specifies that all compatible vector layers should be combined into a single layer instead of maintained in their original layers
            GM_ExportPackage_ExtraElevPrecision = 0x00000080,   // Store elevation values from elevation grid files with extra precision
        };

        // Flags to control the GM_FindNearestFeatures function
        public enum GM_FindFlags_t32 : uint
        {
            GM_FindFlags_FindAreas              = 0x00000001,   // Search area features
            GM_FindFlags_FindLines              = 0x00000002,   // Search line features
            GM_FindFlags_FindPoints             = 0x00000004,   // Search point features
            GM_FindFlags_GetAllFeatures         = 0x00000008,   // Return all features in bounds (ignore search radius)

            // Definitions for masks of various option bits
            GM_FindFlags_FindAll                = 0x00000007,   // Mask for search all feature class (areas, lines, and points)
        };
            
        // Flags describing the type of feature class (i.e. area, point, or line) to work on
        public enum GM_FeatureClassType_t8 : byte
        {
            GM_FeatureClass_Area,       // The feature class is for an area feature
            GM_FeatureClass_Line,       // The feature class is for a line feature
            GM_FeatureClass_Point,      // The feature class is for a point feature

            GM_FeatureClass_NumTypes
        };

        // This type is used to return information about found features
        public struct GM_FoundFeature_t
        {
            public GM_LayerHandle_t32       mLayer;             // Layer the feature is in
            public GM_FeatureClassType_t8   mFeatureClassType;  // Type of feature class (area, point, line)
            public UInt32                   mFeatureIndex;      // Index of feature in layer
            public UInt32                   mDistance;          // Distance in pixels of feature from search location
        };
        
        // Flags to control the behavior of path profile/line of sight analysis
        public enum GM_PathProfileLOSFlags_t32 : uint
        {
            GM_PathProfile_LOSValid             = 0x00000001,   // Perform line-of-sight analysis
            GM_PathProfile_LOSIgnoreEndpoints   = 0x00000002,   // Ignore endpoints when performing line-of-sight analysis (i.e. end points can't block line of sight)
            GM_PathProfile_LOSFromHeightAbsolute= 0x00000004,   // Line-of-sight from height is absolute height above sea level rather than height above ground
            GM_PathProfile_LOSToHeightAbsolute  = 0x00000008,   // Line-of-sight to height is absolute height above sea level rathern than height above ground
            GM_PathProfile_IgnoreEarthCurvature = 0x00000010,   // Ignore earth curvature in any line-of-sight analysis
        };

        // This type is used to provide the parameters for calculating path profiles and 
        // performing line of sight analysis
        public struct GM_PathProfileLOSParams_t
        {
            public UInt32                       mSize;              // IN: Size of structure
            public GM_PathProfileLOSFlags_t32   mFlags;             // IN: Flags controlling behavior
            public double                       mStartX;            // IN: Start X coord in current projection
            public double                       mStartY;            // IN: Start Y coord in current projection
            public double                       mEndX;              // IN: End X coord in current projection
            public double                       mEndY;              // IN: End Y coord in current projection
            public IntPtr /* float* in C */     mElevList;          // OUT: Buffer to hold list of elevations (set to NULL if you don't care). Should be array of 'float'
            public UInt32                       mListSize;          // IN: Number of elevations to retrieve
            public float                        mDfltElev;          // IN: Elev to use when none could be found
            public IntPtr /* char* in C */      mDetailsStr;        // OUT: String containing path details (set to NULL if you don't care)
            public UInt32                       mDetailsStrMaxLen;  // IN: Maximum length of details string
            public double                       mAtmosphericCorr;   // IN: Atmospheric correction for LOS (1.3333 is standard for radio waves, 1.0 is no correction)
            public float                        mLOSFromHeight;     // IN: Line-of-sight from height in meters
            public float                        mLOSToHeight;       // IN: Line-of-sight to height in meters
            public float                        mLOSMinClearance;   // OUT: Minimum line-of-sight clearance in meters (will be negative if the line of sight is not clear)
            public GM_Point_t                   mLOSMinClearancePos;// OUT: Location of minimum line-of-sight clearance
            public double                       mFresnelFreq;       // IN: Frequency (in GHz) to use for testing clearance of first Fresnel zone (use 0.0 to not do Fresnel testing)
            public double                       mFresnelPctClear;   // IN: Percentage of first Fresnel zone that must be clear of obstacles (use 0.6 for 60%). If 0 is specified, the default of 60% is used.
            public GM_Point_t                   mLOSFirstBlockedPos;// OUT: Location of first blocked location along path (only valid if mLOSMinClearance is negative)
            public GM_Point_t                   mLOSLastBlockedPos; // OUT: Location of last blocked location along path (only valid if mLOSMinClearance is negative)
        };

        public enum GM_WebExportFlags_t32 : uint
        {
            GM_WebExport_NoTransparency = 0x00000001,
            GM_WebExport_DisplayResults = 0x00000002,
            GM_WebExport_HideProgress = 0x00000004,
            GM_WebExport_AddScaleBar = 0x00000008,
            GM_WebExport_AddMapTypeControl = 0x00000010,
            GM_WebExport_AddOverviewMap = 0x00000020,
            GM_WebExport_AutoGrid = 0x00000040
        };

        public struct GM_WebFormatExportOpts_t
        {
            public String mMapName;
            public int mMaxZoomLevel;
            public uint mNumZoomLevels;
            public GM_RasterExportFormat_t32 mFormat;
            public GM_WebExportFlags_t32 mFlags;
            public String mApiKey;
            public String mBgMapName;
            public double mTranslucency;
            public double mXRes;
            public double mYRes;
        }

        // Type listing available coordinate delimeters for ASCII import
        public enum GM_AsciiCoordDelim_t8 : byte
        {
            GM_AsciiDelim_AutoDetect    = 0,
            GM_AsciiDelim_Comma         = 1,
            GM_AsciiDelim_Whitespace    = 2,
            GM_AsciiDelim_Semicolon     = 3,
            GM_AsciiDelim_Tab           = 4,

            GM_AsciiDelim_NumTypes
        };

        // Type listing available import types for ASCII import
        public enum GM_AsciiImportType_t8 : byte
        {
            GM_AsciiImport_PointOnly    = 0,    // create point features for every encountered coordinate
            GM_AsciiImport_Vector       = 1,    // create area, line, and point features
            GM_AsciiImport_Elevation    = 2,    // triangulate and grid point cooridnates to generate an elevation grid

            GM_AsciiImport_NumTypes
        };

        // This type represents layout information for a generic ASCII import
        public struct GM_AsciiFormatInfo_t
        {
            public GM_AsciiImportType_t8    mImportType;                // specify import type
            public GM_AsciiCoordDelim_t8    mCoordDelim;                // sepcify coordinate delimeter
            public Byte                     mYCoordFirst;               // is Y coordinate first or X coordinate?
            public Byte                     mIncludeCoordAttrs;         // include attributes from coordinate lines
            public Byte                     mColumnHeadersInFirstRow;   // is the first row of the file column headers? (like for CSV files)
            public Byte                     mIncludeElevCoords;         // is the 3rd coordinate column an elevation or Z value? Must be TRUE for eAsciiImportElevation import type.
            public Byte                     mColumnsToSkip;             // columns to skip before reaching coordinate columns
            public Byte                     mReserved;                  // reserved, must be 0
            public String                   mCoordLinePrefix;           // prefix to look for at beginning of coordinate lines (default to NULL or empty)
            public AreaFeatureClass_t16     mAreaType;                  // area type to assign to imported created area features
            public LineFeatureClass_t16     mLineType;                  // line type to assign to imported created line features
            public PointFeatureClass_t16    mPointType;                 // point type to assign to imported point features
            public Byte                     mIgnoreZeroElevations;      // ignore features with an elevation of zero
            public Byte                     mFillToBounds;              // fill grid to full bounds of data (i.e. don't constrict to convex hull)
            public double                   mOffsetX;                   // offset to apply to X coordinates
            public double                   mOffsetY;                   // offset to apply to Y coordinates
            public double                   mOffsetZ;                   // offset to apply to Z coordinates
            public double                   mScaleX;                    // Scale to apply to X coordinates (default is 1.0)
            public double                   mScaleY;                    // Scale to apply to Y coordinates (default is 1.0)
            public double                   mScaleZ;                    // Scale to apply to Z coordinates (default is 1.0)
            public double                   mNoDataDistanceFactor;      // tightness factor to use when gridding 3D data. Use 0.0 for default.
            public IntPtr /* uint8* */      mBreakOnColumnIdx;          // index of column to break features on when value changes. Pass NULL for default break behavior.
        }

        // Flags to use for adding features to vector layer functions
        public enum GM_AddFeatureFlags_t8 : byte
        {
            GM_AddFeature_UseDefaultStyle   = 0x01, // Use the default style for this feature rather than the specified style
            GM_AddFeature_MoreAddsToFollow  = 0x02, // There will be more features immediately added to this layer, so don't recalculate the bounding box, etc. based on this feature
        };

        public struct GM_ViewShedParams_t
        {
            public UInt32               mSize;              // Size of structure in bytes
            public float                mAngleEnd;          // End angle in radians for radar sweep
            public float                mAngleStart;        // Start angle in radians for radar sweep
            public double               mAtmosphericCorr;   // Atmospheric correction value (1.0 for none)
            public GM_Point_t           mCenterPoint;       // Center point in global coordinates
            public UInt32               mColor;             // (COLORREF) Color to display view shed data in
            public String               mDesc;              // View shed description
            public double               mRadius;            // Search radius in meters
            public IntPtr               mReceiverAngle;     // (double*) Angle (in degrees) above horizon to test at (NULL for none)
            public float                mReceiverHeight;    // Height at which transmitter must be visible
            public Byte                 mReceiverHeightAbs; // Is the receiver height absolute or relative to the ground?
            public Byte                 mShowHiddenPoints;  // Make hidden points visible instead of viewable points.
            public Byte                 mShowProgress;      // Display view shed calculation progress?
            public Byte                 mGenCoveragePolys;  // Generate area/polygon features for the covered area (these are vector and exportable to vector formats)
            public float                mTransmitterHeight; // Transmitter height in meters
            public Byte                 mTransmitterHeightAbs; // Is the transmitter height absolute or relative to the ground?
            public Byte                 mUseEarthCurvature; // Use the curvature of the earth in the analysis?
            public Byte                 mVectorElevsAbs;    // Are vector elevations absolute (i.e. relative to sea level) or relative to ground level?
            public IntPtr               mVectorLayerList;   // (GM_LayerHandle_t32*) List of vector layers to use features with elevation attributes from
            public UInt32               mVectorLayerListCnt;// Number of vector layers in list    
            public double               mXSpacing;          // X sample spacing (use 0.0 for default spacing)
            public double               mYSpacing;          // Y sample spacing (use 0.0 for default spacing)
            public IntPtr               mFilterRect;        // (GM_Rectangle_t*) Optional rectangle for filtering view shed to a certain area of interest
            public IntPtr               mReceiverAngleEnd;  // (double*) Angle (in degrees) above horizon to stop test at (NULL for none) - if this and mReceiverAngle are provided, an angle range view shed test is done (like a flashlight beam)
            public double               mFresnelFreq;       // Frequency (in GHz) to use for testing clearance of first Fresnel zone (use 0.0 to not do Fresnel testing)
            public double               mFresnelPctClear;   // Percentage of first Fresnel zone that must be clear of obstacles (use 0.6 for 60%). If 0 is specified, the default of 60% is used.
        };

		/// <summary>
		/// GlobalMapperDLLInterface
		/// </summary>
		public GlobalMapperDLL()
		{
		}

		/// <summary>
		/// GM_GetSDKVersion
		/// Returns the version of this SDK. The version is obtained by taking the
		/// major version number times 100 and adding the minor version number. For
		/// example, version 1.04 would be returned as 104.
		/// </summary>
		/// <returns>Version for the SDK as an Int32</returns>
		[DllImport(DLLFileName, EntryPoint="GM_GetSDKVersion")]
		public static extern UInt32 GM_GetSDKVersion();

		/// <summary>
		/// Returns a string containing the filter to use in a Windows file open
		/// dialog box for all of the types supported for loading from this DLL.
		/// </summary>
		/// <returns>pointer to a string</returns>
		[DllImport(DLLFileName, EntryPoint="GM_GetFilterString", CharSet=CharSet.Ansi)]
		public static extern IntPtr GM_GetFilterString();

		/// <summary>
		/// GM_LoadGenericAsciiTextLayer
        /// Loads a new layer from a simple ASCII text file file. A pointer to the layer is 
        /// stored for use in later calls.
		/// </summary>
		[DllImport(DLLFileName, EntryPoint="GM_LoadGenericAsciiTextLayer", CharSet=CharSet.Ansi)]
        public static extern GM_Error_t32 GM_LoadGenericAsciiTextLayer
            ( 
            String                      aFilename,  // IN: filename of ASCII file to load
            out GM_LayerHandle_t32      aLayer,     // OUT: layer handle for loaded layer
            GM_LoadFlags_t32            aLoadFlags, // IN: load flags
            ref GM_AsciiFormatInfo_t    aFormatInfo,// IN: optional format info (use NULL to prompt user)
            ref GM_Projection_t         aProj       // IN: optional projection to use for file (use NULL to prompt user)
            );

		/// <summary>
		/// GM_LoadLayer
		/// Loads a new layer from a file. A pointer to the layer is stored for
		/// use in later calls. 
		/// </summary>
		/// <param name="aFilename">Name of the file to open</param>
		/// <param name="aLayer">Returns a handle to a layer</param>
		/// <param name="aLoadFlags">Flags used to load this layer</param>
		/// <returns>Error code of type GM_Error_t32</returns>
		[DllImport(DLLFileName, EntryPoint="GM_LoadLayer")]
		public static extern GM_Error_t32 GM_LoadLayer
        (
            String                  aFilename, 
            out GM_LayerHandle_t32  aLayer, 
            GM_LoadFlags_t32        aLoadFlags
        );

        /// <summary>
        /// GM_LoadLayerList
        /// Loads layers from a file. A pointer to a list of layer handles is returned.
        /// </summary>
        /// <param name="aFilename">Name of the file to open</param>
        /// <param name="aLayerList">Returns a pointer to a list of GM_LayerHandle_t32 values</param>
        /// <param name="aNumLoadedLayers">Returns the number of layers in the list aLayerList points to</param>
        /// <param name="aLoadFlags">Flags used to load this layer</param>
        /// <returns>Error code of type GM_Error_t32</returns>
        [DllImport(DLLFileName, EntryPoint = "GM_LoadLayerList")]
        public static extern GM_Error_t32 GM_LoadLayerList
        (
            String aFilename,
            out IntPtr aLayerList,
            out UInt32 aNumLoadedLayers,
            GM_LoadFlags_t32 aLoadFlags
        );
        
		/// <summary>
		/// GM_LoadOnlineLayer
		/// Loads data from an online source, like a WMS server or TerraServer-USA
		/// </summary>
		[DllImport(DLLFileName, EntryPoint="GM_LoadOnlineLayer")]
        public static extern GM_Error_t32 GM_LoadOnlineLayer
        (
            String                  aSourceName,    // in: name of the TerraServer-USA theme or WMS source to load ("drg", "doq", etc.) or NULL to ask user
            IntPtr                  aLatLonRect,    // in: (const Rectangle2D*) lat/lon rect to import (NULL to ask user)
            out GM_LayerHandle_t32  aLayer,         // out: handle to loaded layer
            IntPtr                  aReserved       // in: reserved parameter (must be NULL)
        );

		/// <summary>
		/// GM_CloseLayer
		/// Closes a previously opened layer.
		/// </summary>
		/// <param name="aLayer">Handle to a currently opened layer</param>
		/// <returns></returns>
		[DllImport(DLLFileName, EntryPoint="GM_CloseLayer")]
		public static extern GM_Error_t32 GM_CloseLayer(IntPtr aLayer);

		/// <summary>
		/// Draws the specified portion of the specified layer to the DIB. If no layer
		/// is explicitly specified, all loaded layers will be drawn.
		/// </summary>
		/// <param name="aDC">Device context to draw to</param>
		/// <param name="aLayer">Layer to draw or NULL for all</param>
		/// <param name="aWorldBounds">World bounds to draw or NULL for all</param>
		/// <param name="aLeftPixel">Left pixel coordinate to draw to</param>
		/// <param name="aTopPixel">Top pixel coordinate to draw to</param>
		/// <param name="aPixelWidth">Width in pixels to draw</param>
		/// <param name="aPixelHeight">Height in pixels to draw</param>
		/// <returns>Error code of type GM_Error_t32</returns>
		[DllImport(DLLFileName, EntryPoint="GM_DrawLayer")]
		public static extern GM_Error_t32 GM_DrawLayer(IntPtr aDC, IntPtr aLayer, ref GM_Rectangle_t aWorldBounds, Int32 aLeftPixel, Int32 aTopPixel, Int32 aPixelWidth, Int32 aPixelHeight);

        /// <summary>
        /// Draws the specified portion of the specified layer(s) to the DC. If no layer
        /// list is explicitly specified, all loaded layers will be drawn.
        /// </summary>
        /// <param name="aDC">Device context to draw to</param>
        /// <param name="aLayerList">List of layers to draw or NULL for all</param>
        /// <param name="aLayerCount">Number of layers in list of layers to draw or 0 for all</param>
        /// <param name="aDrawFlags">Flags to control the drawing</param>
        /// <param name="aWorldBounds">World bounds to draw or NULL for all</param>
        /// <param name="aLeftPixel">Left pixel coordinate to draw to</param>
        /// <param name="aTopPixel">Top pixel coordinate to draw to</param>
        /// <param name="aPixelWidth">Width in pixels to draw</param>
        /// <param name="aPixelHeight">Height in pixels to draw</param>
        /// <returns>Error code of type GM_Error_t32</returns>
        [DllImport(DLLFileName, EntryPoint = "GM_DrawLayerList")]
        public static extern GM_Error_t32 GM_DrawLayerList
            (
            IntPtr aDC,                         // (HDC) Device context to draw to
            IntPtr aLayerList,                  // (GM_LayerHandle_t32*) List of layers to draw or NULL for all
            Int32 aLayerCount,                  // Number of layers in list (0 for all)
            GM_DrawFlags_t32 aDrawFlags,        // Flags controlling how the draw is performed
            ref GM_Rectangle_t aWorldBounds,    // World bounds to draw or NULL for all
            Int32 aLeftPixel,                   // Left pixel coordinate to draw to
            Int32 aTopPixel,                    // Top pixel coordinate to draw to
            Int32 aPixelWidth,                  // Width in pixels to draw
            Int32 aPixelHeight                  // Height in pixels to draw
            );

		/// <summary>
		/// GM_SetProjection
		/// Sets the new projection. This is the projection that all draws and
		/// exports will be done in. It is also the projection that all world bound
		/// rectangles will be in.
		/// </summary>
		[DllImport(DLLFileName, EntryPoint="GM_SetProjection")]
		public static extern GM_Error_t32 GM_SetProjection(ref GM_Projection_t aProj);

		/// <summary>
		/// GM_GetLocationElevation
		/// Retrieves the elevation at a given location from a provided layer or from the
		/// list of currently loaded elevation layers (topmost elevation layer first).
		/// </summary>
		/// <param name="aLayer">IN: Layer to get elevation from or NULL for topmost</param>
		/// <param name="aXCoord">IN: X coord in current projection</param>
		/// <param name="aYCoord">IN: Y coord in current projection</param>
		/// <param name="aElev">OUT: Elevation at location in meters</param>
		/// <returns>Error code of type GM_Error_t32</returns>
		[DllImport(DLLFileName, EntryPoint="GM_GetLocationElevation")]
		public static extern GM_Error_t32 GM_GetLocationElevation(IntPtr aLayer, double aXCoord, double aYCoord, out float aElev);

		/// <summary>
		/// GM_GetLayerInfo
		/// Returns a const pointer to information about the specified layer. Returns
		/// NULL if the layer is not loaded.
		/// </summary>
		/// <param name="aLayer">Handle to a layer for which to retrieve info</param>
		/// <returns>Returns a pointer to information about the specified layer. Returns NULL if the layer is not loaded.</returns>
		[DllImport(DLLFileName, EntryPoint="GM_GetLayerInfo")]
		public static extern IntPtr GM_GetLayerInfo(IntPtr aLayer);

		/// <summary>
		/// Convert a coordinate between the current projection and a pixel coordinate.
		/// The conversion space can be either the last drawn coordinate space or
		/// the caller can provide the coordinate space to work in.
		/// </summary>
		/// <param name="aGroundX">IN: X Coord in current projection</param>
		/// <param name="aGroundY">IN: Y Coord in current projection</param>
		/// <param name="aPixelX">OUT: X Coord in pixel space</param>
		/// <param name="aPixelY">OUT: Y Coord in pixel space</param>
		/// <param name="aWorldBounds">IN: World bounds to convert from or NULL for last drawn.  GM_Rectangle_t*</param>
		/// <param name="aPixelRect">IN: Pixel bounds to convert from or NULL for last drawn.  GM_PixelRect_t*</param>
		/// <returns>Error code of type GM_Error_t32</returns>
		[DllImport(DLLFileName, EntryPoint="GM_ConvertCoordGroundToPixel")]
		public static extern GM_Error_t32 GM_ConvertCoordGroundToPixel(double aGroundX, double aGroundY, out Int32 aPixelX, out Int32 aPixelY, IntPtr aWorldBounds, IntPtr aPixelRect);

		/// <summary>
		/// GM_ConvertCoordPixelToGround
		/// Convert a coordinate between pixel space and the current projection.
		/// The conversion space can be either the last drawn coordinate space or
		/// the caller can provide the coordinate space to work in.
		/// </summary>
		/// <param name="aPixelX">IN: X Coord in pixel space</param>
		/// <param name="aPixelY">IN: Y Coord in pixel space</param>
		/// <param name="aGroundX">OUT: X Coord in current projection</param>
		/// <param name="aGroundY">OUT: Y Coord in current projection</param>
		/// <param name="aWorldBounds">IN: World bounds to convert from or NULL for last drawn.  GM_Rectangle_t*</param>
		/// <param name="aPixelRect">IN: Pixel bounds to convert from or NULL for last drawn.  GM_PixelRect_t*</param>
		/// <returns>Error code of type GM_Error_t32</returns>
		[DllImport(DLLFileName, EntryPoint="GM_ConvertCoordPixelToGround")]
		public static extern GM_Error_t32 GM_ConvertCoordPixelToGround(Int32 aPixelX, Int32 aPixelY, out double aGroundX, out double aGroundY, IntPtr aWorldBounds, IntPtr aPixelRect);
        
		/// <summary>
		/// GM_ExportPackage
		///  Exports one or more layers to a new package file (not implemented on WinCE)
		/// </summary>
		[DllImport( DLLFileName, EntryPoint = "GM_ExportPackage" )]
        public static extern GM_Error_t32
        GM_ExportPackage
        (
            String                      aFilename,      // Name of new file to create
            IntPtr                      aLayerList,     // (GM_LayerHandler_t32*) List of layers to export or NULL for all
            UInt32                      aLayerCount,    // Number of layers in list (0 for all)
            IntPtr                      aWorldBounds,   // (const GM_Rectangle_t*) World bounds to export or NULL for all
            double                      aSimpThreshold, // Simplification threshold, use 0.0 for no simplification
            GM_PackageExportFlags_t32   aFlags          // Export flags
        );

		/// <summary>
		/// GM_ExportRaster
		///  Exports one or more layers to a new raster file (not implemented on WinCE)
		/// </summary>
		[DllImport( DLLFileName, EntryPoint = "GM_ExportRaster" )]
		public static extern GM_Error_t32 
		GM_ExportRaster
		(
			String						aFilename, 
			GM_RasterExportFormat_t32	aFormat, 
			IntPtr						aLayer, 
			ref GM_Rectangle_t			aWorldBounds, 
			Int32						aPixelWidth, 
			Int32						aPixelHeight, 
			GM_RasterExportFlags_t32	aFlags 
		);

        /// <summary> 
        /// GM_ExportWebFormat 
        ///  Exports one or more layers to a files to a web format for display in a web broswer or other ///  web-based tool.
        /// </summary>
        [DllImport(DLLFileName, EntryPoint = "GM_ExportWebFormat")]
        public static extern GM_Error_t32 GM_ExportWebFormat
        (
            String                          aFilename,
            GM_WebExportFormat_t32          aWebFormat,
            IntPtr                          aLayerList,
            UInt32                          aLayerCount,
            ref GM_Rectangle_t              aWorldBounds,
            ref GM_WebFormatExportOpts_t    aExportOpts,
            IntPtr                          aReserved
        );

        /// <summary>
        /// GM_GetPathProfileLOS
        /// Retrieves a list of elevations along a path in the provided layer or from
        /// the list of currently loaded elevation layers (topmost elevation layer first).
        /// Can also perform a line-of-sight calculation and return additional information
        /// about a 3D path (not implemented on WinCE).
        /// </summary>
        [DllImport(DLLFileName, EntryPoint = "GM_GetPathProfileLOS")]
        public static extern GM_Error_t32
        GM_GetPathProfileLOS
            (
            GM_LayerHandle_t32              aLayer, // IN: Layer to get elevation from or NULL for topmost
            ref GM_PathProfileLOSParams_t   aParams // IN/OUT: Calculation parameters
            );

        // Sets the options to use when displaying the given raster layer
        [DllImport(DLLFileName, EntryPoint = "GM_SetRasterDisplayOptions")]
        public static extern GM_Error_t32 GM_SetRasterDisplayOptions
        (
            GM_LayerHandle_t32              aLayer,     // in: layer to set options for
            ref GM_RasterDisplayOptions_t   aOptions    // in: display options for raster layer
        );

        // Retrieves the options to use when displaying the given raster layer
        [DllImport(DLLFileName, EntryPoint = "GM_GetRasterDisplayOptions")]
        public static extern GM_Error_t32 GM_GetRasterDisplayOptions
        (
            GM_LayerHandle_t32              aLayer,     // in: layer to get options for
            out GM_RasterDisplayOptions_t   aOptions    // out: display options for raster layer
        );
        /*--------------------------------------------------------------------
                             VECTOR LAYER QUERY FUNCTIONS
        --------------------------------------------------------------------*/

		/// <summary>
        /// GM_FindNearestFeatures
        /// Finds features within so many pixels of a given pixel location.
        /// </summary>
        [DllImport( DLLFileName, EntryPoint = "GM_FindNearestFeatures" )]
        public static extern GM_Error_t32 
        GM_FindNearestFeatures
            (
            ref GM_Point_t          aPixelSearchPt,     // IN: Search location in pixel coordinates
            IntPtr                  aLayerList,         // IN: List of layers to search or NULL for all vector layers
            UInt32                  aLayerCount,        // IN: Number of layers in list (0 for all)
            GM_FindFlags_t32        aFindFlags,         // IN: Flags controlling how the find is performed
            IntPtr                  aWorldBounds,       // IN: World bounds for search space or NULL for last drawn
            IntPtr                  aPixelRect,         // IN: Pixel bounds for search space or NULL for last drawn 
            IntPtr                  aNearestFeatures,   // OUT: List of nearest features (buffer for GM_FoundFeature_t)
            UInt32                  aMaxFeatures,       // IN: Max number of nearest features to find
            ref UInt32              aFoundFeatureCnt,   // OUT: Number of nearest features returned in list
            UInt32                  aMaxDistPixels      // IN: Max distance in pixels to search
            );

        // Frees the resources used by the given area feature
        [DllImport( DLLFileName, EntryPoint = "GM_FreeAreaFeature" )]
        public static extern void
        GM_FreeAreaFeature
            (
            IntPtr  aAreaFeature            // Area feature to free
            );

        // Frees the resources used by the given line feature
        [DllImport( DLLFileName, EntryPoint = "GM_FreeLineFeature" )]
        public static extern void
        GM_FreeLineFeature
            (
            IntPtr  aLineFeature            // Line feature to free
            );

        // Frees the resources used by the given point feature
        [DllImport( DLLFileName, EntryPoint = "GM_FreePointFeature" )]
        public static extern void
        GM_FreePointFeature
            (
            IntPtr  aPointFeature            // Point feature to free
            );

        // Retrieves the area feature at the given index from the layer
        [DllImport( DLLFileName, EntryPoint = "GM_GetAreaFeature" )]
        public static extern IntPtr
        GM_GetAreaFeature
            (
            GM_LayerHandle_t32  aLayer,                 // Layer to get area from
            UInt32              aAreaIndex              // Index of area feature to retrieve
            );

        // Retrieves the line at the given index from the layer
        [DllImport( DLLFileName, EntryPoint = "GM_GetLineFeature" )]
        public static extern IntPtr
        GM_GetLineFeature
            (
            GM_LayerHandle_t32  aLayer,                 // Layer to get Line from
            UInt32              aLineIndex              // Index of Line feature to retrieve
            );

        // Retrieves the point feature at the given index from the layer
        [DllImport(DLLFileName, EntryPoint = "GM_GetPointFeature")]
        public static extern IntPtr
        GM_GetPointFeature
            (
            GM_LayerHandle_t32  aLayer,                 // Layer to get Point from
            UInt32              aPointIndex             // Index of Point feature to retrieve
            );

        /*--------------------------------------------------------------------
                      CUSTOM LAYER AND FEATURE CREATION FUNCTIONS
        --------------------------------------------------------------------*/

        // Adds an area feature to a loaded vector layer
        [DllImport(DLLFileName, EntryPoint = "GM_AddAreaToVectorLayer")]
        public static extern GM_Error_t32 GM_AddAreaToVectorLayer
            (
            GM_LayerHandle_t32      aLayer,             // IN: Layer to add area to
            ref GM_AreaFeature_t    aArea,              // IN: Area feature to add
            GM_AddFeatureFlags_t8   aFlags              // IN: Flags controlling the add
            );

        // Adds a line feature to a loaded vector layer
        [DllImport(DLLFileName, EntryPoint = "GM_AddLineToVectorLayer")]
        public static extern GM_Error_t32 GM_AddLineToVectorLayer
            (
            GM_LayerHandle_t32      aLayer,             // IN: Layer to add Line to
            ref GM_LineFeature_t    aLine,              // IN: Line feature to add
            GM_AddFeatureFlags_t8   aFlags              // IN: Flags controlling the add
            );

        // Adds a point feature to a loaded vector layer
        [DllImport(DLLFileName, EntryPoint = "GM_AddPointToVectorLayer")]
        public static extern GM_Error_t32 GM_AddPointToVectorLayer
            (
            GM_LayerHandle_t32      aLayer,             // IN: Layer to add Point to
            ref GM_PointFeature_t   aPoint,             // IN: Point feature to add
            GM_AddFeatureFlags_t8   aFlags              // IN: Flags controlling the add
            );

        // Creates a new custom layer for adding vector features to
        [DllImport(DLLFileName, EntryPoint = "GM_CreateCustomVectorLayer")]
        public static extern GM_LayerHandle_t32 GM_CreateCustomVectorLayer
            (
            string                  aDescription,       // IN: Description to use for layer (can be NULL to use default)
            ref GM_Projection_t     aProj               // IN: Native projection of new layer
            );

        /*--------------------------------------------------------------------
                           VECTOR DRAWING STYLE/TYPE FUNCTIONS
        --------------------------------------------------------------------*/

        // Sets the drawing style to use for a given area feature
        [DllImport(DLLFileName, EntryPoint = "GM_SetAreaFeatureDrawStyle")]
        public static extern GM_Error_t32 GM_SetAreaFeatureDrawStyle
            (
            GM_LayerHandle_t32      aLayer,             // Layer the Area is in
            UInt32                  aAreaIndex,         // Index of Area in layer
            IntPtr                  aAreaStyle          // (const GM_AreaStyle_t*) New style to use for Area, pass IntPtr.Zero to restore default type-based stuff
            );

        // Sets the drawing style to use for a given line feature
        [DllImport(DLLFileName, EntryPoint = "GM_SetLineFeatureDrawStyle")]
        public static extern GM_Error_t32 GM_SetLineFeatureDrawStyle
            (
            GM_LayerHandle_t32      aLayer,             // Layer the Line is in
            UInt32                  aLineIndex,         // Index of Line in layer
            IntPtr                  aLineStyle          // (const GM_LineStyle_t*) New style to use for Line, pass IntPtr.Zero to restore default type-based stuff
            );

        // Sets the drawing style to use for a given point feature
        [DllImport(DLLFileName, EntryPoint = "GM_SetPointFeatureDrawStyle")]
        public static extern GM_Error_t32 GM_SetPointFeatureDrawStyle
            (
            GM_LayerHandle_t32      aLayer,             // Layer the point is in
            UInt32                  aPointIndex,        // Index of point in layer
            IntPtr                  aPointStyle         // (const GM_PointStyle_t*) New style to use for point, pass IntPtr.Zero to restore default type-based stuff
            );

        /*--------------------------------------------------------------------
                               SURFACE ANALYSIS FUNCTIONS
        --------------------------------------------------------------------*/

        // Performs a view-shed analysis on loaded elevation data (not implemented on WinCE)
        [DllImport(DLLFileName, EntryPoint = "GM_CalcViewShed")]
        public static extern  GM_Error_t32 GM_CalcViewShed
            (
            GM_LayerHandle_t32          aLayer,         // IN: Layer to get elevations from or NULL for topmost layer at each point
            ref GM_ViewShedParams_t     aViewShedParms, // IN: Parameters for creating view shed
            ref GM_LayerHandle_t32      aViewShedLayer  // OUT: Created view shed layer
            );

        /*--------------------------------------------------------------------
                                    UTILITY FUNCTIONS
        --------------------------------------------------------------------*/

        // Displays a file dialog that allows the user to select a file to load.
        [DllImport(DLLFileName, EntryPoint = "GM_SelectFile")]
        public static extern GM_Error_t32 GM_SelectFile
            ( 
            IntPtr      aFilename,  // buffer in which to place selected filename (must be at least _MAX_PATH characters in length)
            string      aFilterStr, // filter string for open dialog (NULL for default)
            string      aInitialDir,// initial directory to start in (NULL for current)
            IntPtr      aParentWnd  // handle (HWND) to parent window for file selection dialog
            );

        /// <summary>
        /// Select a font. Displays a dialog allowing the user to graphically select a font.
        /// </summary>
        /// <param name="aFont">A line style. IN/OUT: Font buffer - make sure to initialize before passing in</param>
        /// <returns>An Error code.</returns>
        [DllImport(DLLFileName, EntryPoint = "GM_SelectFont")]
        public static extern GM_Error_t32 GM_SelectFont
            (
            ref GM_FontDef_t aFont
            );

        // Message callback function declaration
        public delegate void GM_MessageCallbackDef(string msg);

        // Sets the function to call to display error and warning messages generated
        // during SDK operations. If a message callback is provided, a message
        // dialog will not be shown, instead the callback function will be called
        // with the error or warning message that would have been displayed.
        [DllImport(DLLFileName, EntryPoint = "GM_SetMessageCallback")]
        public static extern void GM_SetMessageCallback
        (
            GM_MessageCallbackDef   aCallBackFunc
        );

        // Message callback function declaration
        public delegate Byte GM_QueryProjectionCallbackDef
            (
            ref GM_Projection_t aProj,          // OUT: projection structure to fill in
            ref GM_Point_t      aInitialPos,    // IN: optional coordinate in projection 
            ref GM_ElevUnits_t8 aElevUnits      // OUT: optional, set to elevation
            );

        [DllImport(DLLFileName, EntryPoint = "GM_SetQueryProjectionCallback")]
        public static extern void GM_SetQueryProjectionCallback
            (
            GM_QueryProjectionCallbackDef   aCallBackFunc
            );

        // Sets the registration name and code to use for this instance of the DLL.
        // This can be used rather than the GMDLL_regkey.txt file to supply the
        // registration information. Will return GM_Error_NotRegistered if the key
        // is not valid for the name.
        [DllImport(DLLFileName, EntryPoint = "GM_SetRegistrationInfo")]
        public static extern GM_Error_t32 GM_SetRegistrationInfo
            (
            string      aRegName,   // name provided for the registration information
            UInt32      aRegKey     // registration key
            );

        // Sets the base registry key from which settings should be stored/read. Settings
        // will be stored in and read from "HKEY_CURRENT_USER\Software\<aKeyName>\Global Mapper".
        [DllImport(DLLFileName, EntryPoint = "GM_SetRegistryKey")]
        public static extern GM_Error_t32 GM_SetRegistryKey
            (
            string      aKeyName    // location in registry under which to store settings
            );
    }
}
