Imports System.Runtime.InteropServices
Imports GMDLLTester_VB.ProjectionTypes
Imports GM_LayerHandle_t32 = System.Int32
Imports AreaFeatureClass_t16 = System.Int16
Imports LineFeatureClass_t16 = System.Int16
Imports PointFeatureClass_t16 = System.Int16
Imports FeatureClass_t16 = System.Int16

Module GlobalMapperInterface

    ' List of supported elevation units
    Public Enum GM_ElevUnits_t8 As Byte
        GM_ElevUnit_Meters = 0
        GM_ElevUnit_Decimeters = 1
        GM_ElevUnit_Centimeters = 2
        GM_ElevUnit_Millimeters = 3
        GM_ElevUnit_Feet = 4
        GM_ElevUnit_Decifeet = 5
        GM_ElevUnit_Inches = 6
        GM_ElevUnit_Centifeet = 7

        GM_ElevUnit_NumUnits
    End Enum

    ' Enumeration of different raster/elevation sampling methods available
    Public Enum GM_SamplingMethod_t8 As Byte
        GM_SAMPLING_NEAREST_NEIGHBOR = 0
        GM_SAMPLING_BILINEAR_INTERPOLATION = 1
        GM_SAMPLING_BICUBIC_INTERPOLATION = 2
        GM_SAMPLING_BOX_AVG_3X3 = 3
        GM_SAMPLING_BOX_AVG_4X4 = 4
        GM_SAMPLING_BOX_AVG_5X5 = 5

        GM_NUM_SAMPLING_METHODS
    End Enum

    ' Flags when loading new layers
    Public Enum GM_LoadFlags_t32 As Integer
        GM_LoadFlags_HideProgress = &H1                 ' Don't show load progress
        GM_LoadFlags_NoProjPrompt = &H2                 ' Don't prompt the user for unknown projections
        GM_LoadFlags_UseDefaultProj = &H4               ' Use default projection if none provided
        GM_LoadFlags_HideWarnings = &H8                 ' Hide any load warnings (load errors will still be displayed)
        GM_LoadFlags_HideUnknownPrompt = &H10           ' Do not prompt the user to select the file type for unknown file formats
        GM_LoadFlags_UseDefaultPos = &H20               ' Use default positioning if none provided (raster files only)
        GM_LoadFlags_HideAllPrompts = &H40              ' Use the default value for any prompts that display during the load
    End Enum

    ' Flags to control the GM_DrawLayerList and GM_DrawPackageList operations
    Public Enum GM_DrawFlags_t32 As Integer
        GM_DrawFlags_BestRasterLayers = &H1                 ' Only draw the best raster layers for this zoom scale
        GM_DrawFlags_LowResRasterLayers = &H2               ' Only draw the lowest resolution raster layers
        GM_DrawFlags_HiResRasterLayers = &H4                ' Only draw the highest resolution raster layers
        GM_DrawFlags_EraseBackground = &H100                ' Erase the background with the current background color (GM_SetBackgroundColor)
        GM_DrawFlags_HideAreas = &H200                      ' Do not draw any area features from vector layers
        GM_DrawFlags_HideLines = &H400                      ' Do not draw any line features from vector layers
        GM_DrawFlags_HidePoints = &H800                     ' Do not draw any point features from vector layers
        GM_DrawFlags_SeparateVectorLayers = &H1000          ' Draw each vector separately rather than drawing all at once.
        GM_DrawFlags_HideLabels = &H2000                    ' Do not label any rendered features
        GM_DrawFlags_DontMaintainAspectRatio = &H4000       ' Do NOT maintain the aspect ratio when drawing (this allows stretched views)
        GM_DrawFlags_HidePartialLabels = &H8000             ' Hide labels that are not entirely in the draw window
        GM_DrawFlags_ShowRenderProgress = &H10000           ' Show render progress dialog or call progress callback if provided
        GM_DrawFlags_HideExtraIconText = &H20000            ' Hide any extra text shown below road icons
        GM_DrawFlags_HideNonIconRoadLabels = &H40000        ' Hide any road line labels that are not iconized
        GM_DrawFlags_RenderVertices = &H80000               ' Render vertices for area and line features
        GM_DrawFlags_IncrementalRender = &H100000           ' Do an incremental render so that draw progress can be seen as layers complete rendering
        GM_DrawFlags_NoAutoInterpolation = &H200000         ' Disable automatic interpolation of rasters at intermediate zoom ranges (results in faster renders w/ less quality)

        ' Definitions for masks of various option bits
        GM_DrawFlags_RasterMask = &H7                   ' Mask for raster draw options
    End Enum

    ' Flags to control the behavior of path profile/line of sight analysis
    Public Enum GM_PathProfileLOSFlags_t32 As Integer
        GM_PathProfile_LOSValid = &H1                       ' Perform line-of-sight analysis
        GM_PathProfile_LOSIgnoreEndpoints = &H2             ' Ignore endpoints when performing line-of-sight analysis (i.e. end points can't block line of sight)
        GM_PathProfile_LOSFromHeightAbsolute = &H4          ' Line-of-sight from height is absolute height above sea level rather than height above ground
        GM_PathProfile_LOSToHeightAbsolute = &H8            ' Line-of-sight to height is absolute height above sea level rathern than height above ground
        GM_PathProfile_IgnoreEarthCurvature = &H10          ' Ignore earth curvature in any line-of-sight analysis
    End Enum

    <StructLayout(LayoutKind.Sequential)> Public Structure GM_ProjAttrValue_t
        Public mAttr As Int16   ' Attribute (see PROJATTR enumeration in ProjectionTypes.h)
        Public mValue As Double ' Attribute value
    End Structure

    <StructLayout(LayoutKind.Sequential)> Public Structure GM_Projection_t
        Public mProjSys As Int16                ' Projection system (see PROJSYS enumeration in ProjectionTypes.h)
        Public mDatum As Int16                  ' Horizontal datum (see DATUM enumeration in ProjectionTypes.h)
        Public mUnit As Int16                   ' Ground units (see UNITS enumeration in ProjectionTypes.h)
        Public mNumAttrs As Int32               ' Number of attributes in attribute list
        Public mAttrVal1 As GM_ProjAttrValue_t  ' Attribute value list (expanded for VB-compatibility)
        Public mAttrVal2 As GM_ProjAttrValue_t
        Public mAttrVal3 As GM_ProjAttrValue_t
        Public mAttrVal4 As GM_ProjAttrValue_t
        Public mAttrVal5 As GM_ProjAttrValue_t
        Public mAttrVal6 As GM_ProjAttrValue_t
        Public mAttrVal7 As GM_ProjAttrValue_t
        Public mAttrVal8 As GM_ProjAttrValue_t
        Public mAttrVal9 As GM_ProjAttrValue_t
        Public mAttrVal10 As GM_ProjAttrValue_t
        Public mAttrVal11 As GM_ProjAttrValue_t
        Public mAttrVal12 As GM_ProjAttrValue_t
        Public mAttrVal13 As GM_ProjAttrValue_t
        Public mAttrVal14 As GM_ProjAttrValue_t
        Public mAttrVal15 As GM_ProjAttrValue_t
        Public mAttrVal16 As GM_ProjAttrValue_t
    End Structure

    <StructLayout(LayoutKind.Sequential)> Public Structure GM_Point_t
        Public mX As Double         ' X (or longitude) coordinate
        Public mY As Double         ' Y (or latitude) coordinate
    End Structure

    <StructLayout(LayoutKind.Sequential)> Public Structure GM_Rectangle_t
        Public mMinX As Double      ' Minimum x/easting/longitude coordinate
        Public mMinY As Double      ' Minimum y/northing/latitude coordinate
        Public mMaxX As Double      ' Maximum x/easting/longitude coordinate
        Public mMaxY As Double      ' Maximum y/northing/latitude coordinate
    End Structure

    <StructLayout(LayoutKind.Sequential)> Public Structure GM_PixelRect_t
        Public mLeft As Int32       ' Left coordinate of rectangle
        Public mRight As Int32      ' Right coordinate of rectangle
        Public mTop As Int32        ' Top coordinate of rectangle
        Public mBottom As Int32     ' Bottom coordinate of rectangle
    End Structure

    <StructLayout(LayoutKind.Sequential)> Public Structure GM_GroundControlPoint_t
        Public mPixelX As Double    ' x pixel coordinate of GCP
        Public mPixelY As Double    ' y pixel coordinate of GCP (top is 0, increases down)
        Public mGroundX As Double   ' x ground coordinate of GCP (in provided projection)
        Public mGroundY As Double   ' y ground coordinate of GCP (in provided projection)
    End Structure

    <StructLayout(LayoutKind.Sequential)> Public Structure GM_LayerInfo_t
        Public mDescription As String           ' Description string
        Public mNativeRect As GM_Rectangle_t    ' Bounding rect of layer in native coordinates
        Public mGlobalRect As GM_Rectangle_t    ' Bounding rect of layer in global coordinates
        Public mPixelWidth As Int32             ' RASTER/ELEV ONLY: Number of pixels wide layer is
        Public mPixelHeight As Int32            ' RASTER/ELEV ONLY: Number of pixels tall layer is
        Public mNativeProj As GM_Projection_t   ' Native projection of layer
        Public mControlPoints As IntPtr         ' RASTER ONLY: Ground control points list
        Public mNumGCPs As Int32                ' RASTER ONLY: Number of ground control points in list
        Public mMinElevation As Single          ' ELEV ONLY: Minimum elevation in meters
        Public mMaxElevation As Single          ' ELEV ONLY: Maximum elevation in meters
        Public mNumAreas As Int32               ' VECTOR ONLY: Number of area features
        Public mNumLines As Int32               ' VECTOR ONLY: Number of line features
        Public mNumPoints As Int32              ' VECTOR ONLY: Number of point features
        Public mPixelSizeX As Double            ' RASTER/ELEV ONLY: Pixel size in meters in the x direction
        Public mPixelSizeY As Double            ' RASTER/ELEV ONLY: Pixel size in meters in the y direction
        Public mHasRasterData As Byte           ' Does this layer have raster or elevation data?
        Public mEnabled As Byte                 ' Is this layer enabled for display or it it hidden?
        Public mHasVectorData As Byte           ' Does this layer have vector data (the features counts can all be 0 for things like map catalogs)
        Public mUsedDefaultPos As Byte          ' RASTER/ELEV ONLY: Was the default position used for this layer since the file could not be automatically positioned?
        Public mFilename As String              ' Filename from which layer was loaded (if this is an archive file, like .zip, then the name of the actual loaded file will be in mArchiveFilename
        Public mArchiveFilename As String       ' Filename within archive file (e.g. .zip or .tar.gz) from which layer was loaded, if any (might be NULL)
        Public mTypeName As String              ' Layer type name
        Public mGlobalPixelWidth As Int32       ' RASTER/ELEV ONLY: Approximate number of pixels required in width for a 1:1 pixel mapping in the current projection
        Public mGlobalPixelHeight As Int32      ' RASTER/ELEV ONLY: Approximate number of pixels required in height for a 1:1 pixel mapping in the current projection
        Public mMetadataList As IntPtr          ' List of metadata attributes and values for the layer (list of GM_AttrValue_t)
        Public mMetadataListSize As Int32       ' Number of entries in mMetadataList
        Public mNumBands As Int32               ' Number of bands in a raster image (use GM_SetRasterDisplayOptions to change color band layout)
        Public mExtraLoadFlags As String        ' Additional load flags with options selected by user during load. Pass these to GM_LoadLayerListEx to remember options.
        Public mPalette As IntPtr               ' GM_PaletteEntry_t* - List of palette entries for palette-based raster layer
        Public mPaletteSize As Int32            ' Number of entries in palette list
        Public mUserData As IntPtr              ' void* - Custom data associated with layer provided by user via GM_SetLayerUserData function
        Public mElevUnits As GM_ElevUnits_t8    ' Elevation units for layer
    End Structure

    ' NOTE: The brush styles are stored in the registry so only add new
    '       syles at the end.
    Public Enum GM_BrushStyle_t16 As Short
        GM_BRUSH_NULL
        GM_BRUSH_SOLID

        ' Hatched brushes
        GM_BRUSH_MIN_HATCH
        GM_BRUSH_BACK_DIAG_CROSSHATCH = GM_BRUSH_MIN_HATCH
        GM_BRUSH_CROSSHATCH
        GM_BRUSH_DIAG_CROSSHATCH
        GM_BRUSH_FWD_DIAG_CROSSHATCH
        GM_BRUSH_HORZ_HATCH
        GM_BRUSH_VERT_HATCH
        GM_BRUSH_MAX_HATCH = GM_BRUSH_VERT_HATCH

        ' Pattern brushes
        GM_BRUSH_MIN_PATTERN
        GM_BRUSH_SAND_PATTERN = GM_BRUSH_MIN_PATTERN
        GM_BRUSH_INT_WATER_PATTERN
        GM_BRUSH_SALT_PATTERN
        GM_BRUSH_CORAL_PATTERN
        GM_BRUSH_BLACK_CROSS_PATTERN
        GM_BRUSH_BLACK_DOT_PATTERN
        GM_BRUSH_MAX_PATTERN = GM_BRUSH_BLACK_DOT_PATTERN

        GM_BRUSH_NUM_STYLES

        ' Custom Symbol support
        GM_BRUSH_FIRST_CUSTOM = &H100
    End Enum

    ' NOTE: The pen styles are stored in the registry so only add new
    '       syles at the end.
    Public Enum GM_PenStyle_t16 As Short
        GM_PEN_SOLID = 0
        GM_PEN_DASH
        GM_PEN_DOT
        GM_PEN_DASHDOT
        GM_PEN_DASHDOTDOT
        GM_PEN_NULL

        ' Stylized lines
        GM_PEN_RAILROAD
        GM_PEN_STRIPED                 ' A solid pen with a single pixel black stripe down the middle

        ' Comb/rake/etc. stylized lines
        GM_PEN_FIRST_DIRECTED
        GM_PEN_LEFT_FACING_LINE = GM_PEN_FIRST_DIRECTED    ' A railroad tie facing to the left of a line
        GM_PEN_RIGHT_FACING_LINE                           ' A railroad tie facing to the right of a line
        GM_PEN_COMB_LONG_TEETH_LEFT                        ' A long railroad tie facing to the left of a line
        GM_PEN_COMB_LONG_TEETH_RIGHT                       ' A long railroad tie facing to the right of a line
        GM_PEN_COMB_WIDE_SPACED_LEFT                       ' Widely spaced railroad ties facing to the left of a line
        GM_PEN_COMB_WIDE_SPACED_RIGHT                      ' Widely spaced railroad ties facing to the right of a line
        GM_PEN_COMB_ALTERNATING_LEFT                       ' Alternating long and short railroad ties to the left of a line
        GM_PEN_COMB_ALTERNATING_RIGHT                      ' Alternating long and short railroad ties to the right of a line
        GM_PEN_COMB_COUPLED_LEFT                           ' Double teeth to the left of a line
        GM_PEN_COMB_COUPLED_RIGHT                          ' Double teeth to the right of a line
        GM_PEN_COMB_TRIPLET_LEFT                           ' Triplet of teeth to the left of a line
        GM_PEN_COMB_TRIPLET_RIGHT                          ' Triplet teeth to the right of a line
        GM_PEN_RAKE_NARROW_SPACED_LEFT                     ' Narrowly space rake of triangles to the left of a line
        GM_PEN_RAKE_NARROW_SPACED_RIGHT                    ' Narrowly space rake of triangles to the right of a line
        GM_PEN_RAKE_LARGE_SPACED_LEFT                      ' Large spaced rake of triangles to the left of a line
        GM_PEN_RAKE_LARGE_SPACED_RIGHT                     ' Large spaced rake of triangles to the right of a line
        GM_PEN_MINUS_SIGNS_LEFT                            ' Minus signs to left of a line
        GM_PEN_MINUS_SIGNS_RIGHT                           ' Minus signs to right of a line
        GM_PEN_PLUS_SIGNS_LEFT                             ' Plus signs to left of a line
        GM_PEN_PLUS_SIGNS_RIGHT                            ' Plus signs to right of a line
        GM_PEN_DOTS_LEFT                                   ' Dots to left of a line
        GM_PEN_DOTS_RIGHT                                  ' Dots to right of a line
        GM_PEN_T_SHAPED_LEFT                               ' T-shape to left of a line
        GM_PEN_T_SHAPED_RIGHT                              ' T-shape to right of a line
        GM_PEN_RECTS_LEFT                                  ' Rectangles to left of a line
        GM_PEN_RECTS_RIGHT                                 ' Rectangles to right of a line
        GM_PEN_SQUARES_LEFT                                ' Squares to left of a line
        GM_PEN_SQUARES_RIGHT                               ' Squares to right of a line
        GM_PEN_HALF_CIRCLE_NARROW_LEFT                     ' Narrow-spaced half circles to the left of a line
        GM_PEN_HALF_CIRCLE_NARROW_RIGHT                    ' Narrow-spaced half circles to the right of a line
        GM_PEN_HALF_CIRCLE_WIDE_LEFT                       ' Widely-spaced half circles to the left of a line
        GM_PEN_HALF_CIRCLE_WIDE_RIGHT                      ' Widely-spaced half circles to the right of a line
        GM_PEN_DIAMONDS_LEFT                               ' Filled diamonds to the left of a line
        GM_PEN_DIAMONDS_RIGHT                              ' Filled diamonds to the right of a line
        GM_PEN_LAST_DIRECTED = GM_PEN_DIAMONDS_RIGHT

        ' Special symbol styles that are directed track of symbols towards one end of the line or the other
        GM_PEN_FIRST_DIRECTED_TRACK = &H40
        GM_PEN_V_TRACK_FORWARD = GM_PEN_FIRST_DIRECTED_TRACK   ' Line made up of V symbols opening away the start of the line
        GM_PEN_V_TRACK_REVERSE                                 ' Line made up of V symbols opening towards the start of the line
        GM_PEN_C_TRACK_FORWARD                                 ' Line made up of C symbols opening away from the start of the line
        GM_PEN_C_TRACK_REVERSE                                 ' Line made up of C symbols opening towards from the start of the line
        GM_PEN_ARROW_TRACK_FORWARD                             ' Line made up of triangles pointing away the start of the line
        GM_PEN_ARROW_TRACK_REVERSE                             ' Line made up of triangles pointing towards the start of the line
        GM_PEN_V_TRACK_SOLID_FORWARD                           ' Line made up of V symbols opening away the start of the solid line
        GM_PEN_V_TRACK_SOLID_REVERSE                           ' Line made up of V symbols opening towards the start of the solid line
        GM_PEN_LAST_DIRECTED_TRACK = GM_PEN_V_TRACK_SOLID_REVERSE

        ' Special symbol styles along solid line
        GM_PEN_FIRST_SOLID_SYMBOL = &H80
        GM_PEN_SOLID_WITH_X = GM_PEN_FIRST_SOLID_SYMBOL ' Solid line with X's through them
        GM_PEN_SOLID_DIAMONDS                          ' Solid line with solid diamonds through them
        GM_PEN_SOLID_EMPTY_DIAMONDS                    ' Solid line with empty diamonds through them
        GM_PEN_SOLID_CIRCLES                           ' Solid line with solid circles through them
        GM_PEN_SOLID_EMPTY_CIRCLES                     ' Solid line with empty circles through them
        GM_PEN_LAST_SOLID_SYMBOL = GM_PEN_SOLID_EMPTY_CIRCLES

        ' Styles with special symbols at end
        GM_PEN_FIRST_END_SYMBOL = &H100
        GM_PEN_ARROW_TO_START = GM_PEN_FIRST_END_SYMBOL    ' Solid line with an arrow pointing to the start of the line
        GM_PEN_ARROW_TO_END                                ' Solid line with an arrow pointing to the end of the line
        GM_PEN_EMPTY_CIRCLE_AT_START                       ' Solid line with an empty circle at the start of the line
        GM_PEN_EMPTY_CIRCLE_AT_END                         ' Solid line with an empty circle at the end of the line
        GM_PEN_LAST_END_SYMBOL = GM_PEN_EMPTY_CIRCLE_AT_END

        ' Additional styles
        GM_PEN_STRIPED_BLACK_EDGES                     ' A solid  pen with a single pixel black stripe down each edge

        GM_PEN_NUM_STYLES
        GM_PEN_FIRST_STYLIZED = GM_PEN_RAILROAD
    End Enum

    ' This type represents the placement of labels relative to a point feature
    Public Enum GM_LabelPlacement_t8 As Byte
        GM_LABEL_AUTO = 0                  ' automatically place label at best location
        GM_LABEL_CENTER = 1                ' center the label on the point
        GM_LABEL_TOP_RIGHT = 2             ' label is up and to the right of point
        GM_LABEL_RIGHT = 3                 ' label is to the right of the point and vertically centered on it
        GM_LABEL_BOTTOM_RIGHT = 4          ' label is below and to the right of point
        GM_LABEL_BOTTOM = 5                ' label is below the point and horizontally centered on it
        GM_LABEL_BOTTOM_LEFT = 6           ' label is below and to the left of point
        GM_LABEL_LEFT = 7                  ' label is left of the point and vertically centered on it
        GM_LABEL_TOP_LEFT = 8              ' label is above and to the left of point
        GM_LABEL_TOP = 9                   ' label is above the point and horizontally centered on it

        GM_LABEL_NUM_PLACEMENTS
    End Enum

    ' This type represents the different font background options
    Public Enum GM_FontBackground_t8 As Byte
        GM_Font_BgAuto = 0          ' Automatically choose transparent or opaque background (all transparent except when centered on line)
        GM_Font_BgOpaque = 1        ' Fill label background with current background color
        GM_Font_BgTransparent = 2   ' Make label background transparent
    End Enum

    <StructLayout(LayoutKind.Sequential)> Public Structure GM_FontDef_t
        Public mFaceName As String                  ' name of font face used
        Public mColor As UInt32                     ' RGB color of font
        Public mWeight As UInt16                    ' weight of font (use Win32 constants, like FW_BOLD, FW_THIN, FW_NORMAL, etc.)
        Public mPointSize As Byte                   ' font point size (set mFixedHgt value to non-zero to use fixed height)
        Public mItalicize As Byte                   ' draw text with italics?
        Public mUnderline As Byte                   ' underline text?
        Public mStrikeout As Byte                   ' strikeout text?
        Public mCharset As Byte                     ' character set (use OEM_CHARSET for default behavior)
        Public mBgMoe As GM_FontBackground_t8       ' background fill mode (i.e. opaque or transparent)
        Public mFixedHgt As Single                  ' fixed height of font in meters (use 0.0 for normal point-size font)
        Public mPlacement As GM_LabelPlacement_t8   ' label placement relative to point features (only applies for point features)
        Public mReserved1 As Byte                   ' reserved, must be 0
        Public mReserved2 As Byte                   ' reserved, must be 0
        Public mReserved3 As Byte                   ' reserved, must be 0
        Public mAngle As Single                     ' counter-clockwise rotation angle of angle (0 is horizontal left to right)
    End Structure

    ' This type is used to describe a style used for drawing an area feature
    ' and its label (if any)
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_AreaStyle_t
        Public mBorderPenStyle As GM_PenStyle_t16   ' border pen style (i.e. solid, dash, etc.)
        Public mBorderPenWidth As UInt16            ' width to draw border pen in
        Public mBorderPenColor As UInt32            ' color to use for border pen
        Public mBrushColor As UInt32                ' color of brush to fill area with (when applicable to brush style). The color is defined as ARGB where the upper 8 bits are alpha. The exception is a value of 0 in the A is treated as no alpha.
        Public mBrushStyle As GM_BrushStyle_t16     ' style of brush to fill area with
        Public mDrawLabel As Byte                   ' render the label for this area if there is one
        Public mDrawLabelAlways As Byte             ' render the label for this area even if it collides with another display label
        Public mFont As GM_FontDef_t                ' font to use to render label
    End Structure

    ' This type is used to describe a style used for drawing a line feature
    ' and its label (if any)
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_LineStyle_t
        Public mPenStyle As GM_PenStyle_t16 ' pen style (i.e. solid, dash, etc.)
        Public mPenWdith As UInt16          ' width in pixels to draw pen in (set to 0 for fixed width lines)
        Public mPenColor As UInt32          ' color to use for pen
        Public mDrawLabel As Byte           ' render the label for this line if there is one
        Public mDrawLabelAlways As Byte     ' render the label for this line even if it collides with another display label
        Public mDrawLabelOnLine As Byte     ' render the label for this line (if any) directly on the line rather than offset from it
        Public mReserved As Byte            ' must be 0
        Public mFont As GM_FontDef_t        ' font to use to render label
        Public mLabelSegment As Int32       ' segment number (1-based) on which to render label (use 0 to use default label placement)
        Public mFixedWidthMeters As Single  ' fixed width in meters to use for pen (only used if mPenWidth is zero)
    End Structure

    ' This type is OR'd with the mRotation member of the GM_PointStyle_t structure to specify
    ' symbol rotation sense.
    Public Enum GM_RotationSense_t16 As UShort
        GM_RotateSense_TopOfMap = &H0  ' Rotation is to top of map (default)
        GM_RotateSense_TrueNorth = &H1000  ' Rotation is relative to true north
        GM_RotateSense_MagNorth = &H2000  ' Rotation is relative to magnetic north

        GM_RotateSense_Mask = &HF000
        GM_RotateAngle_Mask = &HFFF
    End Enum

    ' This type is used to describe a style used for drawing a point feature
    ' and its label (if any)
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_PointStyle_t
        Public mSymbolName As String    ' name of symbol
        Public mFont As GM_FontDef_t    ' font to use to render label
        Public mDrawLabel As Byte       ' render the label for this point if there is one
        Public mDrawLabelAlways As Byte ' render the label for this point even if it collides with another display label
        Public mRotation As UInt16      ' rotation angle in degrees (0 is up/north, 90 right/east, etc.) and rotation sense flags
        Public mScale As Single         ' symbol scaling factor to apply (use negative values to make symbol a fixed height in meters, i.e. -10 would make symbol 10 meters high)
    End Structure

    ' This type is used as the base for any vector feature information
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_VectorFeature_t
        Public mName As String              ' Name of the feature
        Public mDesc As String              ' Description of the feature
        Public mClass As FeatureClass_t16   ' Global Mapper classification assigned to the feature
        Public mAttrList As IntPtr          ' GM_AttrValue_t* - List of attributes associated with feature
        Public mNumAttrs As UInt16          ' Number of attributes in mAttrList
    End Structure

    ' This type is used to store information about holes within area features
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_HoleInArea_t
        Public mPoints As IntPtr    ' GM_Point_t* - List of points in the hole in global coordinates
        Public mNumPoints As UInt32 ' Number of points in the list
    End Structure

    ' This type is used to return information about area features
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_AreaFeature_t
        Public mFeatureInfo As GM_VectorFeature_t   ' General feature info
        Public mPointList As IntPtr                 ' GM_Point_t* - List of points in the area (global coordinates when getting, native when adding feature)
        Public mNumPoints As UInt32                 ' Number of points in the list
        Public mHoleList As IntPtr                  ' GM_HoleInArea_t* - List of holes in the area
        Public mNumHoles As UInt32                  ' Number of holes in the list of holes
        Public mAreaStyle As GM_AreaStyle_t         ' Area render style
        Public mVertexElevList As IntPtr            ' float* - Optional list of elevations for each point in area. If present, must have <mNumPoints> values
    End Structure
    ' This type is used to return information about line features
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_LineFeature_t
        Public mFeatureInfo As GM_VectorFeature_t   ' General feature info
        Public mPointList As IntPtr                 ' GM_Point_t* - List of points in the line (global coordinates when getting, native when adding feature)
        Public mNumPoints As UInt32                 ' Number of points in the list
        Public mLineStyle As GM_LineStyle_t         ' Line render style
        Public mVertexElevList As IntPtr            ' float* - Optional list of elevations for each point in line. If present, must have <mNumPoints> values
    End Structure

    ' This type is used to return information about point features
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_PointFeature_t
        Public mFeatureInfo As GM_VectorFeature_t   ' General feature info
        Public mPos As GM_Point_t                   ' Location of the point (global coordinates when getting, native when adding feature)
        Public mPointStyle As GM_PointStyle_t       ' Point render style (symbol and font)
    End Structure

    ' Flags to control the GM_DrawLayerList and GM_DrawPackageList operations
    Public Enum GM_ContrastMode_t8 As Byte
        GM_Contrast_None = 0
        GM_Contrast_PercentageStretch  ' Stretch within some std-dev of mean to 0-255 range
        GM_Contrast_RangeStretch       ' Stretch entire valid range to 0-255 range

        GM_Contrast_NumModes
        GM_Contrast_DefaultAuto = GM_Contrast_PercentageStretch
    End Enum

    ' This type represents the display options available for raster and elevation layers
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_RasterDisplayOptions_t
        Public mSize As Int32                           ' Size of structure
        Public mContrastMode As GM_ContrastMode_t8      ' Automatically adjust contrast
        Public mAutoClipCollar As Byte                  ' Automatically clip the collar of the DRG layer
        Public mColorIntensity As Byte                  ' Color intensity (0-20). Lower values are lighter, higher values are darker.
        Public mSamplingMethod As GM_SamplingMethod_t8  ' Smooth pixels by using bilinear interpolation.
        Public mTextureMap As Byte                      ' Texture map this raster layer over any underlying elevation layers.
        Public mPad1 As Byte                            ' Pad byte for 8 byte structure alignment (must be 0)
        Public mTranslucency As Int16                   ' Translucency level of layer (0 - completely see-through, 512 - completely opaque)
        Public mTransparent As Byte                     ' Is layer transparent?
        Public mPad2 As Byte                            ' Pad byte for 8 byte structure alignment (must be 0)
        Public mPad3 As Byte                            ' Pad byte for 8 byte structure alignment (must be 0)
        Public mPad4 As Byte                            ' Pad byte for 8 byte structure alignment (must be 0)
        Public mTransparentColor As Int32               ' Color to make transparent
        Public mTransparentColorDist As Byte            ' Fuzzy transparency support. Distance from transparent color to treat nearby colors transparent.
        Public mRedAdjustPercent As Byte                ' Percentage to adjust red color channel by (-100 to 100)
        Public mGreenAdjustPercent As Byte              ' Percentage to adjust red color channel by (-100 to 100)
        Public mBlueAdjustPercent As Byte               ' Percentage to adjust red color channel by (-100 to 100)

        ' Color band layout support
        Public mBandLayoutValid As Byte             ' Are the band layout values valid?
        Public mBandIdxRed As Byte                  ' Index of color band (0-based) to use for red color channel
        Public mBandIdxGreen As Byte                ' Index of color band (0-based) to use for green color channel
        Public mBandIdxBlue As Byte                 ' Index of color band (0-based) to use for blue color channel

        ' Contrast adjustment parameters
        Public mContrastNumStdDev As Double         ' Number of standard deviations from mean to stretch for contrast adjustment (if 0 is used the default of 2 will be used)
        Public mContrastShared As Byte              ' Is the contrast adjustment shared with other loaded layers?
    End Structure

    <StructLayout(LayoutKind.Sequential)> Public Structure GM_ViewShedParams_t
        Public mSize As Int32                   'Size of structure
        Public mAngleEnd As Single              'End angle in radians for radar sweep
        Public mAngleStart As Single            'Start angle in radians for radar sweep
        Public mAtmosphericCorr As Double       'Atmospheric correction value (1.0 for none)
        Public mCenterPoint As GM_Point_t       'Center point in global coordinates
        Public mColor As Int32                  'Color to display view shed data in
        Public mDesc As String                  'View shed description
        Public mRadius As Double                'Search radius in meters
        Public mReceiverAngle As IntPtr         'Angle (in degrees) above horizon to test at (0 for none)
        Public mReceiverHeight As Single        'Height at which transmitter must be visible
        Public mReceiverHeightAbs As Byte       'Is the receiver height absolute or relative to the ground?
        Public mShowHiddenPoints As Byte        'Make hidden points visible instead of viewable points.
        Public mShowProgress As Byte            'Display view shed calculation progress?
        Public mTransmitterHeight As Single     'Transmitter height in meters
        Public mTransmitterHeightAbs As Byte    'Is the transmitter height absolute or relative to the ground?
        Public mUseEarthCurvature As Byte       'Use the curvature of the earth in the analysis?
        Public mVectorElevsAbs As Byte          'Are vector elevations absolute (i.e. relative to sea level) or relative to ground level?
        Public mVectorLayerList As IntPtr       'List of vector layers to use features with elevation attributes from
        Public mVectorLayerListCnt As Int32     'Number of vector layers in list
        Public mXSpacing As Double              'X sample spacing (use 0.0 for default spacing)
        Public mYSpacing As Double              'Y sample spacing (use 0.0 for default spacing)
        Public mFilterRect As IntPtr            'Optional rectangle for filtering view shed to a certain area of interest
        Public mReceiverAngleEnd As IntPtr      'Angle (in degrees) above horizon to stop test at (0 for none) - if this and mReceiverAngle are provided, an angle range view shed test is done (like a flashlight beam)
        Public mFresnelFreq As Double           'Frequency (in GHz) to use for testing clearance of first Fresnel zone (use 0.0 to not do Fresnel testing)
        Public mFresnelPctClear As Double       'Percentage of first Fresnel zone that must be clear of obstacles (use 0.6 for 60%). If 0 is specified, the default of 60% is used.
    End Structure

    ' This type is used to provide the parameters for calculating path profiles and 
    ' performing line of sight analysis
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_PathProfileLOSParams_t
        Public mSize As UInt32
        Public mFlags As GM_PathProfileLOSFlags_t32
        Public mStartX As Double
        Public mStartY As Double
        Public mEndX As Double
        Public mEndY As Double
        Public mElevList As IntPtr      ' Contains address of an array of Single values to fill in
        Public mListSize As UInt32      ' Number of entries to place in mElevList
        Public mDfltElev As Single
        Public mDetailsStr As String
        Public mDetailsStrMaxLen As UInt32
        Public mAtmosphericCorr As Double
        Public mLOSFromHeight As Single
        Public mLOSToHeight As Single
        Public mLOSMinClearance As Single
        Public mLOSMinClearancePos As GM_Point_t
        Public mFresnelFreq As Double
        Public mFresnelPctClear As Double
    End Structure

    ' Type listing available coordinate delimeters for ASCII import
    Public Enum GM_AsciiCoordDelim_t8 As Byte
        GM_AsciiDelim_AutoDetect = 0
        GM_AsciiDelim_Comma = 1
        GM_AsciiDelim_Whitespace = 2
        GM_AsciiDelim_Semicolon = 3
        GM_AsciiDelim_Tab = 4

        GM_AsciiDelim_NumTypes
    End Enum

    ' Type listing available import types for ASCII import
    Public Enum GM_AsciiImportType_t8 As Byte
        GM_AsciiImport_PointOnly = 0    ' create point features for every encountered coordinate
        GM_AsciiImport_Vector = 1       ' create area, line, and point features
        GM_AsciiImport_Elevation = 2    ' triangulate and grid point cooridnates to generate an elevation grid

        GM_AsciiImport_NumTypes
    End Enum

    '// This type represents layout information for a generic ASCII import
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_AsciiFormatInfo_t
        Public mImportType As GM_AsciiImportType_t8     '// specify import type
        Public mCoordDelim As GM_AsciiCoordDelim_t8     '// specify coordinate delimeter
        Public mYCoordFirst As Byte             'boolean// is Y coordinate first or X coordinate?
        Public mIncludeCoordAttrs As Byte       'boolean// include attributes from coordinate lines
        Public mColumnHeadersInFirstRow As Byte 'boolean// is the first row of the file column headers? (like for CSV files)
        Public mIncludeElevCoords As Byte       'boolean// is the 3rd coordinate column an elevation or Z value? Must be TRUE for eAsciiImportElevation import type.
        Public mColumnsToSkip As Byte                   'uint8// columns to skip before reaching coordinate columns
        Public mReserved As Byte                        'uint8// reserved, must be 0
        Public mCoordLinePrefix As String               '// prefix to look for at beginning of coordinate lines (default to NULL or empty)
        Public mAreaType As AreaFeatureClass_t16        '// area type to assign to imported created area features
        Public mLineType As LineFeatureClass_t16        '// line type to assign to imported created line features
        Public mPointType As PointFeatureClass_t16      '// point type to assign to imported point features
        Public mIgnoreZeroElevations As Byte            '// ignore features with an elevation of zero
        Public mFillToBounds As Byte                    '// fill grid to full bounds of data (i.e. don't constrict to convex hull)
        Public mOffsetX As Double                       '// offset to apply to X coordinates
        Public mOffsetY As Double                       '// offset to apply to Y coordinates
        Public mOffsetZ As Double                       '// offset to apply to Z coordinates
        Public mScaleX As Double                        '// Scale to apply to X coordinates (default is 1.0)
        Public mScaleY As Double                        '// Scale to apply to Y coordinates (default is 1.0)
        Public mScaleZ As Double                        '//Scale to apply to Z coordinates (default is 1.0)
        Public mNoDataDistanceFactor As Double          '// tightness factor to use when gridding 3D data. Use 0.0 for default.
        Public mBreakOnColumnIdx As IntPtr              '// index of column to break features on when value changes. Pass NULL for default break behavior.
    End Structure

    ' This type represents a single elevation/color pair for a custom shader
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_ElevColor_t
        Public mElev As Single                          ' Elevation value in meters
        Public mColor As Int32                          ' Color value
    End Structure

    ' This type represents flags available for custom shaders
    Public Enum GM_CustomShaderFlags_t32 As ULong
        GM_CustomShader_DontBlend = &H1                 ' Don't blend colors between elevation values
        GM_CustomShader_StretchToRange = &H2            ' Stretch custom shader to cover valid elevation range
    End Enum

    ' This type represents a custom elevation shader
    <StructLayout(LayoutKind.Sequential)> Public Structure GM_CustomShader_t
        Public mShaderName As String                    ' Name to use for the shader
        Public mFlags As GM_CustomShaderFlags_t32       ' Shader flags
        Public mNumElevs As Int32                       ' Number of elevation/color pairs
        Public mElevColorList As IntPtr                 ' GM_ElevColor_t* - List of elevation/color pairs for shader
    End Structure

    '/*--------------------------------------------------------------------
    '                       LAYER LOAD FUNCTIONS
    '--------------------------------------------------------------------*/

    ' Closes a previously opened layer.
    Public Declare Function GM_CloseLayer Lib "GlobalMapperInterface" _
        ( _
        ByVal aLayer As Int32 _
        ) As Int32

    ' Returns a const pointer to information about the specified layer. Returns
    ' NULL if the layer is not loaded.
    Public Declare Function GM_GetLayerInfo Lib "GlobalMapperInterface" _
        ( _
        ByVal aLayer As Int32 _
        ) As IntPtr

    ' Loads a new layer from a simple ASCII text file file. A pointer to the layer is 
    ' stored for use in later calls.
    Public Declare Function GM_LoadGenericAsciiTextLayer Lib "GlobalMapperInterface" _
        ( _
        ByVal aFilename As String, _
        ByRef aLayer As GM_LayerHandle_t32, _
        ByVal aLoadFlags As Int32, _
        ByRef aFormatInfo As GM_AsciiFormatInfo_t, _
        ByRef aProj As GM_Projection_t _
    ) As Int32

    ' Loads a new layer from a file. A pointer to the layer is stored for
    ' use in later calls. This function is deprecated. You should use
    ' GM_LoadLayerList instead.
    Public Declare Ansi Function GM_LoadLayer Lib "GlobalMapperInterface" _
        ( _
        ByVal aFilename As String, _
        ByRef aLayer As GM_LayerHandle_t32, _
        ByVal aLoadFlags As Int32 _
        ) As Int32

    ' Loads layer(s) from a file. A pointer to a list containing handles
    ' to the loaded layers is returned.
    Public Declare Ansi Function GM_LoadLayerList Lib "GlobalMapperInterface" _
        ( _
        ByVal aFilename As String, _
        ByRef aLayerList As IntPtr, _
        ByRef aNumLoadedLayers As Int32, _
        ByVal aLoadFlags As GM_LoadFlags_t32 _
        ) As Int32

    ' Loads data from the TerraServer, including DRG, DOQ, and urban area imagery
    ' (not implemented on WinCE)
    Public Declare Ansi Function GM_LoadTerraServerLayer Lib "GlobalMapperInterface" _
        ( _
        ByVal aThemeName As String, _
        ByRef aLatLonRect As GM_Rectangle_t, _
        ByRef aLayer As GM_LayerHandle_t32 _
        ) As Int32

    '/*--------------------------------------------------------------------
    '                     VECTOR LAYER QUERY FUNCTIONS
    '--------------------------------------------------------------------*/

    ' Frees the resources used by the given area feature
    Public Declare Sub GM_FreeAreaFeature Lib "GlobalMapperInterface" _
        ( _
        ByVal aAreaFeature As IntPtr _
        )

    ' Frees the resources used by the given line feature
    Public Declare Sub GM_FreeLineFeature Lib "GlobalMapperInterface" _
        ( _
        ByVal aLineFeature As IntPtr _
        )

    ' Frees the resources used by the given point feature
    Public Declare Sub GM_FreePointFeature Lib "GlobalMapperInterface" _
        ( _
        ByVal aPointFeature As IntPtr _
        )

    'Returns the area feature at the given index in the layer. Returns NULL if the layer does not contain an area at the given index or if a registration key was not found and aAreaIndex was greater than GM_MAX_NON_REGISTERED_VECTOR_IDX. 
    Public Declare Function GM_GetAreaFeature Lib "GlobalMapperInterface" _
        ( _
        ByVal aLayerHandle As GM_LayerHandle_t32, _
        ByVal aAreaIndex As UInt32 _
        ) As IntPtr 'GM_AreaFeature_t*

    'Returns the line feature at the given index in the layer. Returns NULL if the layer does not contain a line at the given index or if a registration key was not found and aLineIndex was greater than GM_MAX_NON_REGISTERED_VECTOR_IDX. 
    Public Declare Function GM_GetLineFeature Lib "GlobalMapperInterface" _
        ( _
        ByVal aLayerHandle As GM_LayerHandle_t32, _
        ByVal aLineIndex As UInt32 _
        ) As IntPtr 'GM_LineFeature_t*

    'Returns the point feature at the given index in the layer. Returns NULL if the layer does not contain an point at the given index or if a registration key was not found and apointIndex was greater than GM_MAX_NON_REGISTERED_VECTOR_IDX. 
    Public Declare Function GM_GetPointFeature Lib "GlobalMapperInterface" _
        ( _
        ByVal aLayerHandle As GM_LayerHandle_t32, _
        ByVal aPointIndex As UInt32 _
        ) As IntPtr 'GM_PointFeature_t*

    '/*--------------------------------------------------------------------
    '                VECTOR LAYER EDIT FUNCTIONS
    '--------------------------------------------------------------------*/

    ' Sets the elevation units to use for 3D vector features without an elevation unit in the elevation attribute
    Public Declare Function GM_SetLayerElevationUnits Lib "GlobalMapperInterface" _
        ( _
        ByVal aLayer As GM_LayerHandle_t32, _
        ByVal aElevUnits As GM_ElevUnits_t8 _
        ) As Int32

    '/*--------------------------------------------------------------------
    '                  DISPLAY OPTIONS/DRAWING FUNCTIONS
    '--------------------------------------------------------------------*/

    ' Draws the specified portion of the specified layer to the device context. 
    ' If no layer is explicitly specified, all loaded layers will be drawn.
    Public Declare Function GM_DrawLayer Lib "GlobalMapperInterface" _
        ( _
        ByVal aDC As IntPtr, _
        ByVal aLayer As GM_LayerHandle_t32, _
        ByVal aWorldBounds As IntPtr, _
        ByVal aLeftPixel As Int32, _
        ByVal aTopPixel As Int32, _
        ByVal aPixelWidth As Int32, _
        ByVal aPixelHeight As Int32 _
        ) As Int32

    ' Draws the specified portion of the specified layers to the device context. 
    ' If no layers are explicitly specified, all loaded layers will be drawn.
    Public Declare Function GM_DrawLayerList Lib "GlobalMapperInterface" _
        ( _
        ByVal aDC As IntPtr, _
        ByRef aLayerList As Int32, _
        ByVal aLayerCount As Int32, _
        ByVal aDrawFlags As GM_DrawFlags_t32, _
        ByVal aWorldBounds As IntPtr, _
        ByVal aLeftPixel As Int32, _
        ByVal aTopPixel As Int32, _
        ByVal aPixelWidth As Int32, _
        ByVal aPixelHeight As Int32 _
        ) As Int32

    ' Returns a list of the available elevation shaders (returns GM_CustomShader_t*)
    Public Declare Function GM_GetElevationShaders Lib "GlobalMapperInterface" _
        ( _
        ByRef aNumShaders As Int32, _
        ByVal aCustomOnly As Byte _
        ) As IntPtr

    ' Retrieves the options to use when displaying the given raster layer
    Public Declare Function GM_GetRasterDisplayOptions Lib "GlobalMapperInterface" _
        ( _
        ByVal aLayer As GM_LayerHandle_t32, _
        ByRef aOptions As GM_RasterDisplayOptions_t _
        ) As Int32

    ' Sets the color to be used for background pixels when drawing and exporting
    Public Declare Function GM_SetBackgroundColor Lib "GlobalMapperInterface" _
        ( _
        ByVal aBgColor As Int32 _
        ) As Int32

    ' Allows for drawing data with a direction other than north to the top of
    ' the screen for most projections. A value of 0.0 is the default and
    ' implies no rotation. Technically, the orientation value is the number
    ' of degrees to rotate the rendered data counter-clockwise.
    Public Declare Function GM_SetOrientation Lib "GlobalMapperInterface" _
        ( _
        ByVal aOrientation As Double _
        ) As Int32

    ' Sets the options to use when displaying the given raster layer
    Public Declare Function GM_SetRasterDisplayOptions Lib "GlobalMapperInterface" _
        ( _
        ByVal aLayer As GM_LayerHandle_t32, _
        ByRef aOptions As GM_RasterDisplayOptions_t _
        ) As Int32

    '/*--------------------------------------------------------------------
    '             Coordinate Conversion/Measurement Functions
    '--------------------------------------------------------------------*/

    ' Calculates the Distance Between Two Points 
    '   IN: start X/longitude coordinate
    '   IN: start Y/latitude coordinate
    '   IN: stop X/longitude coordinate
    '   IN: stop Y/latitude coordinate
    '   IN: TRUE - coordinates are lat/lon, FALSE - coordinates are in the current projection?
    '   OUT: distance in meters between the points
    Public Declare Function GM_CalcDistance Lib "GlobalMapperInterface" _
        ( _
        ByVal aFromX As Double, _
        ByVal aFromY As Double, _
        ByVal aToX As Double, _
        ByVal aToY As Double, _
        ByVal aLatLon As Boolean, _
        ByRef aDist As Double _
        ) As Int32

    ' Convert coordinate from pixel to ground coordinates 
    Public Declare Function GM_ConvertCoordPixelToGround Lib "GlobalMapperInterface" _
        ( _
        ByVal aPixelX As Int32, _
        ByVal aPixelY As Int32, _
        ByRef aGroundX As Double, _
        ByRef aGroundY As Double, _
        ByVal aWorldBounds As IntPtr, _
        ByVal aPixelRect As IntPtr _
        ) As Int32

    ' Convert coordinate from ground coordinates to pixel
    Public Declare Function GM_ConvertCoordGroundToPixel Lib "GlobalMapperInterface" _
        ( _
        ByVal aGroundX As Double, _
        ByVal aGroundY As Double, _
        ByRef aPixelX As Int32, _
        ByRef aPixelY As Int32, _
        ByVal aWorldBounds As IntPtr, _
        ByVal aPixelRect As IntPtr _
        ) As Int32

    ' Projects a Point from one Projection/Datum to Another 
    Public Declare Function GM_ProjectPoint Lib "GlobalMapperInterface" _
        ( _
        ByVal aXIn As Double, _
        ByVal aYIn As Double, _
        ByRef aXOut As Double, _
        ByRef aYOut As Double, _
        ByRef aProjIn As GM_Projection_t, _
        ByRef aProjOut As GM_Projection_t _
        ) As Int32

    '/*--------------------------------------------------------------------
    '                      SURFACE ANALYSIS FUNCTIONS
    '--------------------------------------------------------------------*/

    ' Performs a view-shed analysis on loaded elevation data (not implemented on WinCE)
    Public Declare Function GM_CalcViewShed Lib "GlobalMapperInterface" _
          ( _
          ByVal aLayer As GM_LayerHandle_t32, _
          ByRef aViewShedParms As GM_ViewShedParams_t, _
          ByRef aViewShedLayer As GM_LayerHandle_t32 _
          ) As Int32

    ' Retrieves a list of elevations along a path in the provided layer or from
    ' the list of currently loaded elevation layers (topmost elevation layer first).
    ' Can also perform a line-of-sight calculation and return additional information
    ' about a 3D path (not implemented on WinCE).
    Public Declare Function GM_GetPathProfileLOS Lib "GlobalMapperInterface" _
        ( _
        ByVal aLayer As Int32, _
        ByRef aParams As GM_PathProfileLOSParams_t _
        ) As Int32

    '/*--------------------------------------------------------------------
    '                         UTILITY FUNCTIONS
    '--------------------------------------------------------------------*/

    ' Retrieves the global projection that is being used for all
    ' rendering and exporting.
    Public Declare Function GM_GetProjection Lib "GlobalMapperInterface" _
        ( _
        ByRef aProj As GM_Projection_t _
        ) As Int32

    ' Returns the version of this SDK. The version is obtained by taking the
    ' major version number times 100 and adding the minor version number. For
    ' example, version 1.04 would be returned as 104.
    Public Declare Function GM_GetSDKVersion Lib "GlobalMapperInterface" () As Int32

    ' Sets the new projection. This is the projection that all draws and
    ' exports will be done in. It is also the projection that all world bound
    ' rectangles will be in.
    Public Declare Function GM_SetProjection Lib "GlobalMapperInterface" _
        ( _
        ByVal aProj As IntPtr _
        ) As Int32

End Module
